/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : master3.c
              Author : S.R.Christian, D. Boreham
   Date first issued : 27-Oct-91



Date        Modification                                    Initials
----        ------------                                    --------
27-Oct-91   Adapted from original                           SRC
******************************************************************/
/*}}}*/

#include <stdio.h>
#include <stdlib.h>
#include <misc.h>
#include <string.h>
#include <channel.h>
#include <process.h> 
#include "vme.h"
#include "basic.h"

#define  blocksize 0x100000
#define  MEGABYTE  0x0100000
#define  BYTEPAT   (unsigned int)0x01234567
#define  CLEARPAT  (unsigned int)0xaaaaaaaa



void printspeed(t1,t2)
int t1,t2;
{
    printf ("Data Moved at %f bytes/s\n",(15625.0*blocksize)/ProcTimeMinus(t2,t1));
}   

int main()
{
  Channel *from_slave;
  Channel *to_slave;
  int i=0;
  unsigned int *addr;
  unsigned int *dram;
  unsigned int *d32,*d16b01,*d16b23,*d08b0,*d08b1,*d08b2,*d08b3;

  int t1,t2;

    /* slave board has made all 4M RAM accessible in 24 bit address space
       at 0x100000 Thus it will be repeated in 256 locations in 32 bit
       address space. We take advantage of this to define different master
       access byte multiplex patterns for reading and writing to same area*/

    dram =    (unsigned int *)0x80100000; 
    d32  =    (unsigned int *)0x00100000;   
    d16b01 =  (unsigned int *)0x04100000;   
    d16b23 =  (unsigned int *)0x05100000;   
    d08b0  =  (unsigned int *)0x06100000;   
    d08b1  =  (unsigned int *)0x07100000;   
    d08b2  =  (unsigned int *)0x08100000;   
    d08b3  =  (unsigned int *)0x09100000;   

    from_slave = (Channel *)get_param(3); /* get the channel addresses */
    to_slave   = (Channel *)get_param(4); /* from the cfs file.        */
    
    printf("Starting Tests\n");

    /* initialise board as slot 1 master */
    basic_board_init();
    i=init_slot_one(ARBITER_PRI,32);
    i=init_vme_master(3,RELMODE_ROR,1); /* set writepost to true */  
                                  
                       
    program_byte_mux(1,0x32103210, 0x32103210);
    program_byte_mux(2,0x76547654, 0x76547654);
    program_byte_mux(3,0x76767676, 0x76767676);
    program_byte_mux(4,0x54545454, 0x54545454);
    program_byte_mux(5,0x32323232, 0x32323232);
    program_byte_mux(6,0x10101010, 0x10101010); 
    turnon_byte_mux();

    i=prog_master_access(d32,MEGABYTE,32,24,0,0);

    i=prog_master_access(d16b01,MEGABYTE,16,24,0,1);
    i=prog_master_access(d16b23,MEGABYTE,16,24,2,2);

    i=prog_master_access(d08b0,MEGABYTE,8,24,0,3);
    i=prog_master_access(d08b1,MEGABYTE,8,24,1,4);
    i=prog_master_access(d08b2,MEGABYTE,8,24,2,5);
    i=prog_master_access(d08b3,MEGABYTE,8,24,3,6); 


    i=turnon_vme_master();

    i = ChanInInt(from_slave);                /* Get stuff from slave     */
    ChanOutInt(to_slave,1);                   /* & send to the slave      */
                                              /* So the boards are synced */
                                              /* & talking to each other. */

    printf ("Boards setup & synchronised\n");

    addr = dram;
    for(i = 0; i <(MEGABYTE/4);i++)
       *addr++ = BYTEPAT;
    t1 = ProcTime();
    memcpy(d32, dram, MEGABYTE);
    t2 = ProcTime();
    printspeed(t1,t2);

    printf("Multiplexed access...\n\n");

    printf("D32, read %X\n",*d32);
    printf("D16 Byte 0-1, read %X\n",*d16b01);
    printf("D16 Byte 2-3, read %X\n",*d16b23);  
    printf("D08, byte 0 read %X\n",*d08b0);     
    printf("D08, byte 1 read %X\n",*d08b1);   
    printf("D08, byte 2 read %X\n",*d08b2); 
    printf("D08, byte 3 read %X\n",*d08b3);
    ProcWait(10000); 
                      
    /*{{{16 bit access*/
    t1 = ProcTime();
    memcpy(d16b01, dram, MEGABYTE);
    t2 = ProcTime();
    printspeed(t1,t2);
    
    printf("D32, read %X\n",*d32);
    printf("D16 Byte 0-1, read %X\n",*d16b01);
    printf("D16 Byte 2-3, read %X\n",*d16b23);  
    printf("D08, byte 0 read %X\n",*d08b0);     
    printf("D08, byte 1 read %X\n",*d08b1);   
    printf("D08, byte 2 read %X\n",*d08b2); 
    printf("D08, byte 3 read %X\n",*d08b3);
    ProcWait(10000); 
    /*}}}*/
    /*{{{16 bit access*/
    t1 = ProcTime();
    memcpy(d16b23, dram, MEGABYTE);
    t2 = ProcTime();
    printspeed(t1,t2);
    
    printf("D32, read %X\n",*d32);
    printf("D16 Byte 0-1, read %X\n",*d16b01);
    printf("D16 Byte 2-3, read %X\n",*d16b23);  
    printf("D08, byte 0 read %X\n",*d08b0);     
    printf("D08, byte 1 read %X\n",*d08b1);   
    printf("D08, byte 2 read %X\n",*d08b2); 
    printf("D08, byte 3 read %X\n",*d08b3);
    ProcWait(10000); 
    /*}}}*/
    /*{{{8 bit access*/
    t1 = ProcTime();
    memcpy(d08b0, dram, MEGABYTE);
    t2 = ProcTime();
    printspeed(t1,t2);
    
    printf("D32, read %X\n",*d32);
    printf("D16 Byte 0-1, read %X\n",*d16b01);
    printf("D16 Byte 2-3, read %X\n",*d16b23);  
    printf("D08, byte 0 read %X\n",*d08b0);     
    printf("D08, byte 1 read %X\n",*d08b1);   
    printf("D08, byte 2 read %X\n",*d08b2); 
    printf("D08, byte 3 read %X\n",*d08b3);
    ProcWait(10000); 
    /*}}}*/
    /*{{{8 bit access*/
    t1 = ProcTime();
    memcpy(d08b1, dram, MEGABYTE);
    t2 = ProcTime();
    printspeed(t1,t2);
    
    printf("D32, read %X\n",*d32);
    printf("D16 Byte 0-1, read %X\n",*d16b01);
    printf("D16 Byte 2-3, read %X\n",*d16b23);  
    printf("D08, byte 0 read %X\n",*d08b0);     
    printf("D08, byte 1 read %X\n",*d08b1);   
    printf("D08, byte 2 read %X\n",*d08b2); 
    printf("D08, byte 3 read %X\n",*d08b3);
    ProcWait(10000); 
    /*}}}*/
    /*{{{8 bit access*/
    t1 = ProcTime();
    memcpy(d08b2, dram, MEGABYTE);
    t2 = ProcTime();
    printspeed(t1,t2);
    
    printf("D32, read %X\n",*d32);
    printf("D16 Byte 0-1, read %X\n",*d16b01);
    printf("D16 Byte 2-3, read %X\n",*d16b23);  
    printf("D08, byte 0 read %X\n",*d08b0);     
    printf("D08, byte 1 read %X\n",*d08b1);   
    printf("D08, byte 2 read %X\n",*d08b2); 
    printf("D08, byte 3 read %X\n",*d08b3);
    ProcWait(10000); 
    /*}}}*/
    /*{{{8 bit access*/
    t1 = ProcTime();
    memcpy(d08b3, dram, MEGABYTE);
    t2 = ProcTime();
    printspeed(t1,t2);
    
    printf("D32, read %X\n",*d32);
    printf("D16 Byte 0-1, read %X\n",*d16b01);
    printf("D16 Byte 2-3, read %X\n",*d16b23);  
    printf("D08, byte 0 read %X\n",*d08b0);     
    printf("D08, byte 1 read %X\n",*d08b1);   
    printf("D08, byte 2 read %X\n",*d08b2); 
    printf("D08, byte 3 read %X\n",*d08b3);
    ProcWait(10000); 
    /*}}}*/


    exit_terminate (0);     

}

