/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : master4.c
              Author : S.R.Christian, D. Johnston
   Date first issued : 12-Apr-91



Date        Modification                                    Initials
----        ------------                                    --------
02-Oct-91   Adapted from original                           SRC
******************************************************************/
/*}}}*/

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <misc.h>
#include <time.h>
#include "vme.h"
#include "basic.h"


/* Addresses into which the A16 region for the analog card will be mapped */
#define BYTE0 0x1500000
#define BYTE1 0x2500000
#define BYTE2 0x3500000
#define BYTE3 0x4500000
#define WORD0 0x5500000
#define WORD1 0x6500000

/* Global pointers to the card base and card registers. */
uint *xvme0=(uint *)BYTE0;  /* points to start of board, byte 0 */
uint *xvme1=(uint *)BYTE1;  /* points to start of board, byte 1 */
uint *xvme2=(uint *)BYTE2;  /* points to start of board, byte 2 */
uint *xvme3=(uint *)BYTE3;  /* points to start of board, byte 3 */
uint *xvmewd0=(uint *)WORD0; /* point to start of board, word 0 */
uint *xvmewd1=(uint *)WORD1; /* point to start of board, word 1 */
uint *status=(uint *)(BYTE1+0x80);  /* points at status reg             */
uint *chan=(uint *)(BYTE1+0x84);    /* points at channel/gain reg */
uint *adc=(uint *)(WORD1+0x84);     /* points at adc reg */

void setup_mapram(void)
{
  program_byte_mux(1,0x54545454,0x10101010); /* Program in the swap function */
                                             /* Lower byte on write mapped   */
                                             /* to all bytes.*/
                                             /* Third byte on read mapped   */
                                             /* to all bytes.*/
  program_byte_mux(2,0x76547654,0x32103210); /* Program in the swap function */

  prog_master_access((void *)BYTE0,0x00100000,8  ,16 ,0, 1);
  prog_master_access((void *)BYTE1,0x00100000,8  ,16 ,1, 1);
  prog_master_access((void *)BYTE2,0x00100000,8  ,16 ,2, 1);
  prog_master_access((void *)BYTE3,0x00100000,8  ,16 ,3, 1);

  prog_master_access((void *)WORD0,0x00100000,16 ,16 ,0, 2);
  prog_master_access((void *)WORD1,0x00100000,16 ,16 ,2, 2);
}

void set_green_light(void)
{
    *status=0x03; /* set the green light */
}

void read_id_rom(void)
{
  int i;
  char c;
    printf("\n");
    for(i=0;i<=10;i++)
      {
      c=(char)((xvme1[i]) & 0xff);  /* Read a character. */
      printf("%c",c);               /* Print it. */
      c=(char)((xvme3[i]) & 0xff);  /* Read character in next 16 bit word */
      printf("%c",c);               /* Print it. */
      }
    printf("\n");
}

void read_adc(void)
{
  int adc_val;
    *chan=0x20;  /* set channel 0 , gain 1 */
    while (TRUE)
      {
        *status=0xCA;   /* set both lights, force a conversion */
        ProcWait(2);    /* Wait for conversion to complete */
        adc_val=(*adc) & 0xFFFF;            /* Read conversion result */
        printf("%c%8.8x",(char)13,adc_val); /* Display it */
      }
}

int main()
{
  int i;

    basic_board_init();                  /* initialise board */
    i=init_slot_one(ARBITER_PRI,32);     /* Slot one functions */

    /* Bus capture and hold mode. 64us timeout before BERR, writepost off */
    i= init_vme_master(RELMODE_BCAP,VMETIMEOUT_64US, NO_WRITEPOST);
 
    setup_mapram();
    i=turnon_vme_master();

    turnon_byte_mux();
    set_green_light();                   /* doit */
    read_id_rom();
    read_adc();
    exit_terminate(0);                   /* exit gracefully */
}
