/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990,1991

    Source file name : clilib.h
              Author : S.R.Christian
   Date first issued : 12-Jun-90


This module contains the constant and type declaration for the cli code

Date        Modification                                    Initials
----        ------------                                    --------
12-Jun-90   Created module from existing BCPL harness       SRC
********************************************************************/
/*}}}*/

typedef short boolean;     /* define a logical type value */
#define true  1
#define false 0

/* more explicit character definitions */

#define space_char  0x20
#define tab_char    0x09
#define escape_char '\\'
#define nl_char     '\n'
#define cr_char     '\r'

/* key return value from integer operations that fail */

/*#define bad_integer_val  0x80000000 */
#define bad_integer_val  (long) 0x80000000

/* control flow construct identifier values */

#define no_const     0
#define define_const 1
#define if_const     2
#define else_const   3
#define while_const  4
#define for_const    5

/* Tokeniser constants */
/*{{{*/
#define max_tok_len 80
#define no_token    0

/* tokeniser data structure */

struct parse {
  char    *line_p;   /* string under examination */
  short   par_pos;   /* index of current position, at delimiter or EOL */
  short   par_sta;   /* index of start position for last operation */
  short   tok_len;   /* length of identified token */
  char    tok_del;   /* delimiter of current token (not part of token) */
  char    token[max_tok_len];  /* actual token found, in upper case */
};

typedef struct parse parse_t; 
/*}}}*/

/* macro store structure */
/*{{{*/
struct macro {
  struct macro   *line_p;
  char           line[max_tok_len];
};

typedef struct macro macro_t;
/*}}}*/

/* symbol table data structure and types */
/*{{{*/

#define no_symbol  0
#define int_symbol 1       /* integer symbol */
#define flt_symbol 2       /* floating point symbol */
#define str_symbol 4       /* string symbol */
#define com_symbol 8       /* command symbol */
#define mac_symbol 16      /* macro symbol */

#define any_symbol 0xff    /* matches all symbol types */

struct symtab {
  char     *name_p;  /* symbol id */
  short    type;     /* type of symbol */
  short    name_len; /* length of symbol name */
  union    {
    long    int_val;
    double  flt_val;
    char    *str_val;
    boolean (*com_val)(parse_t*, char*);
    macro_t *mac_val;
  }        value;   /* value of symbol */
  boolean  fixed;   /* flag for symbol */
  short    depth;   /* nesting depth at which declaration made */
  char     *info_p; /* informational string */
};

typedef struct symtab symtab_t;
/*}}}*/

/* function prototypes for user */
/*{{{*/
void print(const char* format, ...);
boolean is_matched(char *tested_p, char *definition_p, short minlen);
long conv_int(char *token_p, short default_base);
boolean conv_flt(char *token_p, double *value_p);
void tag_current_line(parse_t *pars_p, char *message_p);
void init_pars(parse_t *pars_p, char *new_line_p);
void cp_pars(parse_t *dest_p, parse_t *source_p);
short get_tok(parse_t *pars_p, char *delim_p);
boolean assign_integer(char *token_p, long value, boolean constant);
boolean assign_float(char *token_p, double value, boolean constant);
boolean assign_string(char *token_p, char *value, boolean constant);
boolean register_command(char *token_p, boolean (*action)(parse_t*, char*),
                         char *help_p);
boolean evaluate_integer(char *token_p, long *value_p, short default_base);
boolean evaluate_float(char *token_p, double *value_p);
boolean evaluate_comparison(char *token_p, boolean *result_p,
                            short default_base);
boolean evaluate_string(char *token_p, char *string_p, short max_len);
boolean cget_string(parse_t *pars_p, char *default_p, char *result_p, short max_len);
boolean cget_integer(parse_t *pars_p, long def_int, long *result_p);
boolean cget_float(parse_t *pars_p, double def_flt, double *result_p);
boolean cget_item(parse_t *pars_p, char *default_p, char *result_p, short max_len);
void cli_main(boolean (*setup_r)(), long max_symbols, short default_base,
              char *ip_prompt_p, char *file_p);
/*}}}*/


