/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : duart.h
              Author : S.R.Christian, D. Johnston
   Date first issued : 12-Apr-91

Definitions to support IMS B016 DUART

Date        Modification                                    Initials
----        ------------                                    --------
02-Oct-91   Adapted from original                           SRC
******************************************************************/
/*}}}*/
/*{{{DUART REGS*/
#define DUART                    0X7FD80000
#define DUART_MR1A               0X7FD80000
#define DUART_MR2A               0X7FD80000
#define DUART_SRA                0X7FD80004
#define DUART_CSRA               0X7FD80004
#define DUART_CRA                0X7FD80008
#define DUART_THRA               0X7FD8000C
#define DUART_RHRA               0X7FD8000C
#define DUART_AUX                0X7FD80010
#define DUART_ACR                0X7FD80010
#define DUART_IPCR               0X7FD80010
#define DUART_IMR                0X7FD80014
#define DUART_ISR                0X7FD80014
#define DUART_CTUR               0X7FD80018
#define DUART_CTLR               0X7FD8001C
#define DUART_MR1B               0X7FD80020
#define DUART_MR2B               0X7FD80020
#define DUART_CSRB               0X7FD80024
#define DUART_SRB                0X7FD80024
#define DUART_CRB                0X7FD80028
#define DUART_THRB               0X7FD8002C
#define DUART_RHRB               0X7FD8002C
#define DUART_OPCR               0X7FD80034
#define DUART_INPUT_PORT         0X7FD80034
#define DUART_SET_OP_PORT_BITS   0X7FD80038
#define DUART_START_COUNTER      0X7FD80038
#define DUART_RESET_OP_PORT_BITS 0X7FD8003C
#define DUART_STOP_COUNTER       0X7FD8003C
/*}}}*/

/*{{{SETUP DECLARATIONS*/
#define DUART_TX_BAUD_A          1
#define DUART_TX_BAUD_B          2
#define DUART_RX_BAUD_A          3
#define DUART_RX_BAUD_B          4
#define DUART_DATA_BITS_A        5
#define DUART_DATA_BITS_B        6
#define DUART_STOP_BITS_A        7
#define DUART_STOP_BITS_B        8
#define DUART_PARITY_A           9
#define DUART_PARITY_B           10
#define DUART_ENABLE_TX_A        11
#define DUART_ENABLE_TX_B        12
#define DUART_ENABLE_RX_A        13
#define DUART_ENABLE_RX_B        14
#define DUART_RESET_ERROR_A      15
#define DUART_RESET_ERROR_B      16
#define DUART_RESET_TX_A         17
#define DUART_RESET_RX_A         18
#define DUART_RESET_TX_B         19
#define DUART_RESET_RX_B         20
/*}}}*/

#define DUART_ODD                1
#define DUART_EVEN               2
#define DUART_NONE               3


#define DUART_BUFFSIZE_A 256          /* BUFFER SIZES */
#define DUART_BUFFSIZE_B 256
#define DUART_INBUFFSIZE_A 256
#define DUART_INBUFFSIZE_B 256
#define DUART_MARGIN_A 10           /* HEADROOM BEFORE XON/XOFF SENT */
#define DUART_MARGIN_B 10
#define XON 0X11
#define XOFF 0X13
#define EMPTYING 0
#define FILLING 1


extern int duart_setup(int parameter,int value);
extern void duart_init(void);
extern void duart_use_xon_a(int use);
extern void duart_use_xon_b(int use);
extern int duart_putchar_a(char c);
extern char duart_getchar_a(void);
extern int duart_putchar_b(char c);
extern char duart_getchar_b(void);
extern void duart_write_a(char *text);
extern void duart_write_b(char *text);

#define BAUD_50     1
#define BAUD_75     2
#define BAUD_110    3
#define BAUD_134.5  4
#define BAUD_150    5
#define BAUD_200    6
#define BAUD_300    7
#define BAUD_600    8
#define BAUD_1050   9
#define BAUD_1200   10
#define BAUD_1800   11
#define BAUD_2000   12
#define BAUD_2400   13
#define BAUD_4800   14
#define BAUD_7200   15
#define BAUD_9600   16
#define BAUD_19.2k  17
#define BAUD_38.4k  18

/* number of data bits can be 5,6,7 or 8 */
/* take number of stop bits from the table: */
/*
entry  number of stop bits
1      1
2      2
3      0.563
4      0.625
5      0.688
6      0.750
7      0.813
8      0.875
9      0.938
10     1
11     1.563
12     1.625
13     1.688
14     1.750
15     1.813
16     1.875
17     1.938
18     2
*/

/* Take baud rate from the table: */
/*
entry    baud rate
1        50
2        75
3        110
4        134.5
5        150
6        200
7        300
8        600
9        1050
10       1200
11       1800
12       2000
13       2400
14       4800
15       7200
16       9600
17       19.2k
18       38.4k
*/
