/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : duart.c
              Author : S.R.Christian, D. Johnston
   Date first issued : 12-Apr-91

Support routines for duart control on IMS B016

Date        Modification                                    Initials
----        ------------                                    --------
02-Oct-91   Adapted from original                           SRC
06-Oct-91   Added carriage return stuffing to write routine SRC
******************************************************************/
/*}}}*/

#include <stdlib.h>
#include <process.h>
#include <misc.h>
#include "basic.h"
#include "duart.h"
#include "event.h"

/*{{{static variables (shared between processes)*/
static int duart_head_a;          /* shared vars for buffers */
static int duart_head_b;
static int duart_tail_a;
static int duart_tail_b;
static int duart_inhead_a;          /* shared vars for buffers */
static int duart_inhead_b;
static int duart_intail_a;
static int duart_intail_b;
static int duart_rxflag_a;         /* xon/xoff flags */
static int duart_txflag_a;
static int duart_rxflag_b;
static int duart_txflag_b;
static int duart_usex_a;          /* xon being used flag */
static int duart_usex_b;          /* xon being used flag */
static char duart_buffer_a[DUART_BUFFSIZE_A];         /* buffers */
static char duart_buffer_b[DUART_BUFFSIZE_B];
static char duart_inbuffer_a[DUART_INBUFFSIZE_A];
static char duart_inbuffer_b[DUART_INBUFFSIZE_B];
static Channel *duart_rx_a;
static Channel *duart_rx_b;
static int duart_int_mask;                            /* interrupt mask */
/*}}}*/

/*{{{duart_write*/
void duart_write_a(char *text)
{
  /* use a generic routine, but for every '\n' character seen, we add a
     '\r' as well. This makes programming I/O easier for UNIX hackers */
  int i;

  for (i=0;text[i]!='\0'; i++) {
    if (text[i] == '\n')
      duart_putchar_a('\r');
    duart_putchar_a(text[i]);
  }
}
 
void duart_write_b(char *text)
{
  /* use a generic routine, but for every '\n' character seen, we add a
     '\r' as well. This makes programming I/O easier for UNIX hackers */
  int i;

  for (i=0;text[i]!='\0'; i++) {
    if (text[i] == '\n')
      duart_putchar_b('\r');
    duart_putchar_b(text[i]);
  }
}

/*}}}*/
/*{{{duart_buffer full&empty*/
int duart_is_buffer_almost_full_a(void)
  {
  int space=0;
  int data=0;
  int full=FALSE;
  int hd,tl;

    hd=duart_inhead_a;
    tl=duart_intail_a;
    if (hd==tl)
      {space=0; }
    else
      if (hd>tl)
        {
          data=(hd-tl);
          space=(DUART_INBUFFSIZE_A-1)-data;
        }
      else
        if (hd<tl)
          {
            space=tl-hd;
          }
    if (space < DUART_MARGIN_A) {full = TRUE;}
    else {full=FALSE;}
    return(full);
  }
 
/****int duart_is_buffer_almost_full_b(void);*/
int duart_is_buffer_almost_full_b(void)
  {
  int space=0;
  int data=0;
  int full=FALSE;
  int hd,tl;

    hd=duart_inhead_b;
    tl=duart_intail_b;
    if (hd==tl)
      {space=0; }
    else
      if (hd>tl)
        {
          data=(hd-tl);
          space=(DUART_INBUFFSIZE_B-1)-data;
        }
      else
        if (hd<tl)
          {
            space=tl-hd;
          }
    if (space < DUART_MARGIN_B) {full = TRUE;}
    else {full=FALSE;}
    return(full);
  }
 
/****int duart_is_buffer_almost_empty_a(void);*/
int duart_is_buffer_almost_empty_a(void)
  {
  int space=0;
  int data=0;
  int empty=FALSE;
  int hd,tl;

    hd=duart_inhead_a;
    tl=duart_intail_a;
    if (hd==tl)
      { space=0; }
    else
      if (hd>tl)
        {
          data=(hd-tl);
          space=(DUART_INBUFFSIZE_A-1)-data;
        }
      else
        if (hd<tl)
          {
            space=tl-hd;
          }

    if (space > ((DUART_INBUFFSIZE_A-1) - DUART_MARGIN_A)) {empty = TRUE;}
    else {empty=FALSE;}
    return(empty);
  }
 
/****int duart_is_buffer_almost_empty_b(void);*/
int duart_is_buffer_almost_empty_b(void)
  {
  int space=0;
  int data=0;
  int empty=FALSE;
  int hd,tl;

    hd=duart_inhead_b;
    tl=duart_intail_b;
    if (hd==tl)
      { space=0; }
    else
      if (hd>tl)
        {
          data=(hd-tl);
          space=(DUART_INBUFFSIZE_B-1)-data;
        }
      else
        if (hd<tl)
          {
            space=tl-hd;
          }

    if (space > ((DUART_INBUFFSIZE_B-1) - DUART_MARGIN_B)) {empty = TRUE;}
    else {empty=FALSE;}
    return(empty);
  }
/*}}}*/
/*{{{duart_putchar*/
int duart_putchar_a(char c)
  {
  int hd,tl;
  int done=FALSE;
  int mask;
    hd=duart_head_a;
    duart_buffer_a[hd]=c;
    hd+=1;
    if (hd==DUART_BUFFSIZE_A) {hd=0;}
    duart_head_a=hd;

      /****Wait for space in the buffer*/
      while (done==FALSE)
        {
          hd=duart_head_a;          /* Optimiser takes these out of loop */
          tl=duart_tail_a;
            if (hd==DUART_BUFFSIZE_A) {hd=0;}
            if (tl==DUART_BUFFSIZE_A) {tl=0;}
            if (hd!=tl)
              {
                done=TRUE;
              }
            else
              {
                hd=hd;
                tl=tl;
              }          /* Therefore fool the optimiser */
        }
       

   if ((duart_rxflag_a==XON) || (duart_usex_a==FALSE))
     {
       mask=(duart_int_mask | 0x01);
       duart_int_mask=mask;
       *((uint *)DUART_IMR)=mask; /* reenable the tx interrupt */
     }
  }
 
/****int duart_putchar_b(char c);*/
int duart_putchar_b(char c)
  {
  int hd,tl;
  int done=FALSE;
  int mask;
    hd=duart_head_b;
    duart_buffer_b[hd]=c;
    hd+=1;
    if (hd==DUART_BUFFSIZE_B) {hd=0;}
    duart_head_b=hd;

      /****Wait for space in the buffer*/
      while (done==FALSE)
        {
          hd=duart_head_b;          /* Optimiser takes these out of loop */
          tl=duart_tail_b;
            if (hd==DUART_BUFFSIZE_B) {hd=0;}
            if (tl==DUART_BUFFSIZE_B) {tl=0;}
            if (hd!=tl)
              {
                done=TRUE;
              }
            else
              {
                hd=hd;
                tl=tl;
              }          /* Therefore fool the optimiser */
        }
       

   if ((duart_rxflag_b==XON) || (duart_usex_b==FALSE))
     {
       mask=(duart_int_mask | 0x10);
       duart_int_mask=mask;
       *((uint *)DUART_IMR)=mask; /* reenable the tx interrupt */
     }
  }
/*}}}*/
 
/*{{{duart_sendchar*/
void duart_sendchar_a(void)
  {
  int hd;
  int tl;
  char c;
  int mask;

    /****the output routine*/
    hd=duart_head_a;
    tl=duart_tail_a;
    
    if  ((duart_is_buffer_almost_empty_a()==TRUE)
         && (duart_txflag_a==EMPTYING)
         && (duart_usex_a==TRUE))     /* if using xon/xoff send xon */
      {
        /* send xon char */
        *((uint *)DUART_THRA) = XON;  /* send the char, clear the tx interrupt */
        duart_txflag_a=FILLING;
      }
    else
      {
        if  ((duart_is_buffer_almost_full_a()==TRUE)
            && (duart_txflag_a==FILLING)
            && (duart_usex_a==TRUE))  /* if using xon/xoff send xoff */
          {
            /* send xoff char */
            *((uint *)DUART_THRA) = XOFF;  /* send the char, clear the tx interrupt */
            duart_txflag_a=EMPTYING;
          }
        else
          {
            if ((!((hd==(tl+1)) || ((hd==0) && (tl==(DUART_BUFFSIZE_A-1))))) /*if not empty */
                & ((duart_rxflag_a==XON) || (duart_usex_a==FALSE)))
              {
                tl+=1;
                  if (tl==DUART_BUFFSIZE_A) {tl=0;}
                c = duart_buffer_a[tl];
                *((uint *)DUART_THRA) = c;    /* send the char, clear the tx interrupt */
                duart_tail_a=tl;
              }
            else   /* else buffer is empty so wait for putchar() to wake us up */
              {
                mask=(duart_int_mask & 0x32);  /* buffer empty. Disable int */
                duart_int_mask=mask;
                *((uint *)DUART_IMR)=mask;  /* disable the tx interrupt */
              }
          }
      }
    
     
  }
 
/****void duart_sendchar_b(void);*/
void duart_sendchar_b(void)
  {
  int hd;
  int tl;
  char c;
  int mask;

    /****the output routine*/
    hd=duart_head_b;
    tl=duart_tail_b;
    
    if  ((duart_is_buffer_almost_empty_b()==TRUE)
         && (duart_txflag_b==EMPTYING)
         && (duart_usex_b==TRUE))     /* if using xon/xoff send xon */
      {
        /* send xon char */
        *((uint *)DUART_THRA) = XON;  /* send the char, clear the tx interrupt */
        duart_txflag_b=FILLING;
      }
    else
      {
        if  ((duart_is_buffer_almost_full_b()==TRUE)
            && (duart_txflag_b==FILLING)
            && (duart_usex_b==TRUE))  /* if using xon/xoff send xoff */
          {
            /* send xoff char */
            *((uint *)DUART_THRB) = XOFF;  /* send the char, clear the tx interrupt */
            duart_txflag_b=EMPTYING;
          }
        else
          {
            if ((!((hd==(tl+1)) || ((hd==0) && (tl==(DUART_BUFFSIZE_B-1))))) /*if not empty */
                & ((duart_rxflag_b==XON) || (duart_usex_b==FALSE)))
              {
                tl+=1;
                  if (tl==DUART_BUFFSIZE_B) {tl=0;}
                c = duart_buffer_b[tl];
                *((uint *)DUART_THRB) = c;    /* send the char, clear the tx interrupt */
                duart_tail_b=tl;
              }
            else   /* else buffer is empty so wait for putchar() to wake us up */
              {
                mask=(duart_int_mask & 0x23);  /* buffer empty. Disable int */
                duart_int_mask=mask;
                *((uint *)DUART_IMR)=mask;  /* disable the tx interrupt */
              }
          }
      }
    
     
  }
/*}}}*/
/*{{{duart_getchar*/
char duart_getchar_a(void)
  {
  int hd,tl;
    hd=duart_inhead_a;
    tl=duart_intail_a;
    while (((hd==(tl+1)) ||
            ((hd==0) && (tl==(DUART_BUFFSIZE_A-1))))) /* while empty */
        {
          hd=duart_inhead_a;          /* Optimiser takes these out of loop */
          tl=duart_intail_a;
          hd=hd;
          tl=tl;                      /* so fool the optimiser */
        }
    tl+=1;                            /* take char off back of buffer */
    if (tl==DUART_INBUFFSIZE_A) {tl=0;}
    duart_intail_a=tl;
    return(duart_inbuffer_a[tl]);
  }
 
char duart_getchar_b(void)
  {
  int hd,tl;
    hd=duart_inhead_b;
    tl=duart_intail_b;
      while (((hd==(tl+1)) ||
            ((hd==0) && (tl==(DUART_BUFFSIZE_B-1))))) /* while empty */
        {
          hd=duart_inhead_b;          /* Optimiser takes these out of loop */
          tl=duart_intail_b;
          hd=hd;
          tl=tl;                      /* so fool the optimiser */
        }
    tl+=1;                            /* take char off back of buffer */
    if (tl==DUART_INBUFFSIZE_B) {tl=0;}
    duart_intail_b=tl;
    return(duart_inbuffer_b[tl]);
  }
/*}}}*/
/*{{{duart_takechar*/
void duart_takechar_a(Process *p)
  {
  int hd;
  int tl;
  char c;
  int mask;
    p=p;
    for(;;)
      {
        c=ChanInChar(duart_rx_a);     /* get the char */
        hd=duart_inhead_a;
        tl=duart_intail_a;
        duart_inbuffer_a[hd]=c;   /* put it in the buffer */
        if ((((duart_is_buffer_almost_empty_a()==TRUE)
          || (duart_is_buffer_almost_full_a()==TRUE))) && (duart_usex_a==TRUE))
          {
            mask=(duart_int_mask | 0x01);
            duart_int_mask=mask;
            *((uint *)DUART_IMR)=mask; /* reenable the tx interrupt */
            /* enable tx interrupts so that xoff or xon can be sent. */
          }
        while (hd==tl)            /* Wait until not full */
          {
            hd=duart_inhead_a;
            tl=duart_intail_a;
            hd=hd;
            tl=tl;                   /* Fool the optimiser */
          }

        hd+=1;                    /* shuffle the buffer pointers */
        if (hd==DUART_INBUFFSIZE_A) {hd=0;}
        duart_inhead_a=hd;
      }
  }
 
/****void duart_takechar_b(void);*/
void duart_takechar_b(Process *p)
  {
  int hd;
  int tl;
  char c;
  int mask;
    p=p;
    for(;;)
      {
        c=ChanInChar(duart_rx_b);     /* get the char */
        hd=duart_inhead_b;
        tl=duart_intail_b;
        duart_inbuffer_b[hd]=c;   /* put it in the buffer */
        if ((((duart_is_buffer_almost_empty_b()==TRUE)
          || (duart_is_buffer_almost_full_b()==TRUE))) && (duart_usex_b==TRUE))
          {
            mask=(duart_int_mask | 0x10);
            duart_int_mask=mask;
            *((uint *)DUART_IMR)=mask; /* reenable the tx interrupt */
            /* enable tx interrupts so that xoff or xon can be sent. */
          }
        while (hd==tl)            /* Wait until not full */
          {
            hd=duart_inhead_b;
            tl=duart_intail_b;
            hd=hd;
            tl=tl;                   /* Fool the optimiser */
          }

        hd+=1;                    /* shuffle the buffer pointers */
        if (hd==DUART_INBUFFSIZE_B) {hd=0;}
        duart_inhead_b=hd;
      }
  }
/*}}}*/

/*{{{duart_event*/
void duart_event_handler(void)
  {
  int mask;
  int isr;
  int c;
    isr=(*((uint *)DUART_ISR)) & 0xFF;     /* Get interrupt status */
    if (isr != 0)
      /****Identify what caused the interrupt & wakeup the buffer routines*/
      {
        isr=(isr & duart_int_mask);
        if ((isr & 0x01)==0x01)  /* Then transmitter A is ready */
           
          {
            if ((duart_rxflag_a==XOFF) && (duart_usex_a==TRUE))
              {
                mask=(duart_int_mask & 0x32);
                duart_int_mask=mask;
                *((uint *)DUART_IMR)=mask; /* disable the tx interrupt */
              }
            else
              {
                duart_sendchar_a();  /* send a char from the buffer */
              }
          }
           
        if ((isr & 0x10)==0x10)  /* Then transmitter B is ready */
           
          {
            if ((duart_rxflag_b==XOFF) && (duart_usex_b==TRUE))
              {
                mask=(duart_int_mask & 0x23);
                duart_int_mask=mask;
                *((uint *)DUART_IMR)=mask; /* disable the tx interrupt */
              }
            else
              {
                duart_sendchar_b();  /* send a char from the buffer */
              }
          }
           
        if ((isr & 0x02)==0x02)  /* Then char recieved on port A */
           
          {
            c=(*((uint *)DUART_RHRA)) & 0xFF;     /* get the char, clear the int */
            if (((c==(char)XON) || (c==(char)XOFF)) && (duart_usex_a==TRUE))
              {
                duart_rxflag_a=(c & 0x7F);
                if (c==(char)XON)
                  {
                    mask=(duart_int_mask | 0x01);
                    duart_int_mask=mask;
                    *((uint *)DUART_IMR)=mask; /* enable the tx interrupt */
                  }
                else
                  {
                    mask=(duart_int_mask & 0x32);
                    duart_int_mask=mask;
                    *((uint *)DUART_IMR)=mask; /* disable the tx interrupt */
                  }
              }
            else
              {
                ChanOutChar(duart_rx_a,(char)c);      /* wakeup recieve routine */
              }
          }
           
        if ((isr & 0x20)==0x20)  /* Then char recieved on port B */
           
          {
            c=(*((uint *)DUART_RHRB)) & 0xFF;     /* get the char, clear the int */
            if (((c==(char)XON) || (c==(char)XOFF)) && (duart_usex_b==TRUE))
              {
                duart_rxflag_b=(c & 0x7F);
                if (c==(char)XON)
                  {
                    mask=(duart_int_mask | 0x10);   /**/
                    duart_int_mask=mask;
                    *((uint *)DUART_IMR)=mask; /* enable the tx interrupt */
                  }
                else
                  {
                    mask=(duart_int_mask & 0x23);    /**/
                    duart_int_mask=mask;
                    *((uint *)DUART_IMR)=mask; /* disable the tx interrupt */
                  }
              }
            else
              {
                ChanOutChar(duart_rx_b,(char)c);      /* wakeup recieve routine */
              }
          }
           
      }
       
  }
/*}}}*/
/*{{{duart_init*/
void duart_init(void)
  {
  Process *p;
    duart_head_a = 1; /* empty buffers */
    duart_inhead_a = 1; /* empty buffers */
    duart_head_b = 1;
    duart_inhead_b = 1;
    duart_tail_a = 0;
    duart_intail_a = 0;
    duart_tail_b = 0;
    duart_intail_b = 0;
    duart_usex_a=FALSE;     /* xon/xoff flags */
    duart_usex_b=FALSE;
    duart_rxflag_a=XON;
    duart_rxflag_b=XON;
    duart_txflag_a=FILLING;
    duart_txflag_b=FILLING;

    duart_int_mask=0x00;
    *((uint *)DUART_IMR)=duart_int_mask;  /* Interrupts off */
    /****allocate channels & processes*/
    
    if ((duart_rx_a = ChanAlloc()) == NULL)
      {
        exit(-1);
      }
    if ((duart_rx_b = ChanAlloc()) == NULL)
      {
        exit(-1);
      }
    
    ChanInit(duart_rx_a);
    ChanInit(duart_rx_b);
    
    if ((p=ProcAlloc(duart_takechar_a,0,0))==NULL)
      {
        exit(-1);
      }
    
    ProcRunHigh(p); /* start the process */
    
    if ((p=ProcAlloc(duart_takechar_b,0,0))==NULL)
      {
        exit(-1);
      }
    
    ProcRunHigh(p); /* start the process */
     

    *((uint *)DUART_CRA)=0x20; /* reset reciever a */
    *((uint *)DUART_CRA)=0x30; /* reset transmitter a */
    *((uint *)DUART_CRA)=0x40; /* reset error status a*/
    *((uint *)DUART_CRA)=0x50; /* reset chan a break change */

    *((uint *)DUART_CRB)=0x20; /* reset reciever b */
    *((uint *)DUART_CRB)=0x30; /* reset transmitter b */
    *((uint *)DUART_CRB)=0x40; /* reset error status b */
    *((uint *)DUART_CRB)=0x50; /* reset chan b break change */

    *((uint *)DUART_CRA) = 0x10;     /* point at m1ra */
    (*((uint *)DUART_MR1A))=0x12;    /* set RxRDY rather than FFULL */

    *((uint *)DUART_CRB) = 0x10;     /* point at m1ra */
    (*((uint *)DUART_MR1B))=0x12;    /* set RxRDY rather than FFULL */

    install_duart_int_handler(duart_event_handler);

    duart_int_mask=0x33;  /* enable input & output interrupts */
    *((uint *)DUART_IMR)=duart_int_mask;

  }
/*}}}*/
/*{{{duart_use_xon*/
void duart_use_xon_a(int use)
  {
    if (use==TRUE)
      {
        duart_usex_a=TRUE;
        duart_txflag_a=FILLING;
        duart_rxflag_a=XON;
      }
    else
      {
        duart_usex_a=FALSE;
      }
  }
 
/****void duart_use_xon_b(TRUE|FALSE)*/
void duart_use_xon_b(int use)
  {
  if (use==TRUE)
      {
        duart_usex_b=TRUE;
        duart_txflag_b=FILLING;
        duart_rxflag_b=XON;
      }
    else
      {
        duart_usex_b=FALSE;
      }
  }
/*}}}*/
 
/*{{{setup utilities*/
int duart_get_stop_bits(int value)
  {
  static int stop_table[19]={0,7,15,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
    return stop_table[value];
  }
 
int duart_get_baud_bits(int value)
  {
  static int baud_table[19]={0,0,0,1,2,3,3,4,5,7,6,10,7,8,9,10,11,12,12};
    return baud_table[value];
  }
 
void duart_brg_set(int value)
  {
    if ((value==2) ||
        (value==5) ||
        (value==11) ||
        (value==12) ||
        (value==17))
        {
          *((uint *)DUART_ACR)=0x80; /* baud rate set 1 */
        }
    else
        {
          *((uint *)DUART_ACR)=0x00; /* baud rate set 0 */
        }
  }
/*}}}*/
 
/*{{{duart_setup*/
int duart_setup(int parameter,int value)
  {
  static int baud_a;
  static int baud_b;

  int baud;
  int mr1a;
  int mr2a;
  int mr1b;
  int mr2b;
    switch (parameter)
       
      {
        case DUART_TX_BAUD_A   :
           
          duart_brg_set(value);
          baud=duart_get_baud_bits(value);
          baud_a=((baud & 0x0F) | (baud_a & 0xF0));
          *((uint *)DUART_CSRA)=baud_a;
          break;
           
        case DUART_TX_BAUD_B   :
           
            duart_brg_set(value);
            baud=duart_get_baud_bits(value);
            baud_b=((baud & 0x0F) | (baud_b & 0xF0));
            *((uint *)DUART_CSRB)=baud_b;
            break;
           
        case DUART_RX_BAUD_A   :
           
            duart_brg_set(value);
            baud=((duart_get_baud_bits(value)) << 4);
            baud_a=((baud & 0xF0) | (baud_a & 0x0F));
            *((uint *)DUART_CSRA)=baud_a;
            break;
           
        case DUART_RX_BAUD_B   :
           
            duart_brg_set(value);
            baud=(duart_get_baud_bits(value) << 4);
            baud_b=((baud & 0xF0) | (baud_b & 0x0F));
            *((uint *)DUART_CSRB)=baud_b;
            break;
           
        case DUART_DATA_BITS_A :
           
            value-=5;
            *((uint *)DUART_CRA) = 0x10;     /* point at m1ra */
            mr1a = *((uint *)DUART_MR1A);
            mr1a = ((mr1a & 0xFC) | (value & 0x03));
            *((uint *)DUART_CRA) = 0x10;     /* point at m1ra */
            (*((uint *)DUART_MR1A))=mr1a;
            break;
           
        case DUART_DATA_BITS_B :
           
            value-=5;
            *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
            mr1b = *((uint *)DUART_MR1B);
            mr1b = ((mr1b & 0xFC) | (value & 0x03));
            *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
            (*((uint *)DUART_MR1B))=mr1b;
            break;
           
        case DUART_STOP_BITS_A :
           
            value=duart_get_stop_bits(value);
            *((uint *)DUART_CRA) = 0x10;     /* point at mr1a */
            mr1a = *((uint *)DUART_MR1A);    /* point at mr2a */
            mr2a = (value & 0x0F);
            (*((uint *)DUART_MR2A))=mr2a;
            break;
           
        case DUART_STOP_BITS_B :
           
            value=duart_get_stop_bits(value);
            *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
            mr1b = *((uint *)DUART_MR1B);    /* point at mr2b */
            mr2b = *((uint *)DUART_MR2B);
            mr2b = (value & 0x0F);
            (*((uint *)DUART_MR2B))=mr2b;
            break;
           
        case DUART_PARITY_A    :
           
            switch (value)
              {
                case DUART_EVEN:
                    *((uint *)DUART_CRA) = 0x10;     /* point at mr1a */
                    mr1a = *((uint *)DUART_MR1A);
                    mr1a = (mr1a & 0xE3);            /* set to even parity */
                    *((uint *)DUART_CRA) = 0x10;     /* point at mr1a */
                    (*((uint *)DUART_MR1A))=mr1a;
                    break;
                case DUART_ODD:
                    *((uint *)DUART_CRA) = 0x10;     /* point at mr1a */
                    mr1a = *((uint *)DUART_MR1A);
                    mr1a = ((mr1a & 0xE3) | 0x04);   /* set to odd parity */
                    *((uint *)DUART_CRA) = 0x10;     /* point at mr1a */
                    (*((uint *)DUART_MR1A))=mr1a;
                    break;
                case DUART_NONE:
                    *((uint *)DUART_CRA) = 0x10;     /* point at mr1a */
                    mr1a = *((uint *)DUART_MR1A);
                    mr1a = ((mr1a & 0xE3) | (0x10));
                    *((uint *)DUART_CRA) = 0x10;     /* point at mr1a */
                    (*((uint *)DUART_MR1A))=mr1a;
                    break;
                default: {}
              }
            break;
           
        case DUART_PARITY_B    :
           
            switch (value)
              {
                case DUART_EVEN:
                    *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
                    mr1b = *((uint *)DUART_MR1B);
                    mr1b = (mr1b & 0xE3);            /* set to even parity */
                    *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
                    (*((uint *)DUART_MR1B))=mr1b;
                    break;
                case DUART_ODD:
                    *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
                    mr1b = *((uint *)DUART_MR1B);
                    mr1b = ((mr1b & 0xE3) | 0x04);   /* set to odd parity */
                    *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
                    (*((uint *)DUART_MR1B))=mr1b;
                    break;
                case DUART_NONE:
                    *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
                    mr1b = *((uint *)DUART_MR1B);
                    mr1b = ((mr1b & 0xE3) | (0x10)); /* turn off parity */
                    *((uint *)DUART_CRB) = 0x10;     /* point at mr1b */
                    (*((uint *)DUART_MR1B))=mr1b;
                    break;
                default : {}
              }
            break;
           
        case DUART_ENABLE_TX_A :
           
            if (value)
              *((uint *)DUART_CRA) = 0x04;     /* no command, enable tx */
            else
              *((uint *)DUART_CRA) = 0x08;     /* no command, disable tx */
            break;
           
        case DUART_ENABLE_TX_B :
           
            if (value)
              *((uint *)DUART_CRB) = 0x04;     /* no command, enable tx */
            else
              *((uint *)DUART_CRB) = 0x08;     /* no command, disable tx */
            break;
           
        case DUART_ENABLE_RX_A :
           
            if (value)
              *((uint *)DUART_CRA) = 0x01;     /* no command, enable rx */
            else
              *((uint *)DUART_CRA) = 0x02;     /* no command, disable rx */
            break;
           
        case DUART_ENABLE_RX_B :
           
            if (value)
              *((uint *)DUART_CRB) = 0x01;     /* no command, enable rx */
            else
              *((uint *)DUART_CRB) = 0x02;     /* no command, disable rx */
            break;
           
        case DUART_RESET_ERROR_A :
           
            *((uint *)DUART_CRA) = 0x40;     /* Reset Tx */
            break;
           
        case DUART_RESET_TX_A  :
           
            *((uint *)DUART_CRA) = 0x30;     /* Reset Tx */
            break;
           
        case DUART_RESET_RX_A  :
           
            *((uint *)DUART_CRA) = 0x20;     /* Reset Rx */
            break;
           
        case DUART_RESET_ERROR_B :
           
            *((uint *)DUART_CRB) = 0x40;     /* Reset Tx */
            break;
           
        case DUART_RESET_TX_B  :
           
            *((uint *)DUART_CRB) = 0x30;     /* Reset Tx */
            break;
           
        case DUART_RESET_RX_B  :
           
            *((uint *)DUART_CRB) = 0x20;     /* Reset Rx */
            break;
           
        default:
           
            return (-1);
            break;
           
      }
       
  return(0);
  }
/*}}}*/
 
/*{{{duart_quick_setup_9600*/
void duart_quick_setup_9600(void)
  {

    duart_setup(DUART_RX_BAUD_B,17);    /* 9600 baud */
    duart_setup(DUART_TX_BAUD_B,77);    /* 9600 baud */
    duart_setup(DUART_RESET_ERROR_B,0);
    duart_setup(DUART_ENABLE_TX_B,FALSE);
    duart_setup(DUART_ENABLE_RX_B,FALSE); /* Reset error, disable rx & tx */
    duart_setup(DUART_RESET_TX_B,0);   /* Reset Tx */
    duart_setup(DUART_RESET_RX_B,0);   /* Reset Rx */
    duart_setup(DUART_DATA_BITS_B,8);
    duart_setup(DUART_STOP_BITS_B,1);
    duart_setup(DUART_PARITY_B,DUART_NONE);
    duart_setup(DUART_ENABLE_TX_B,TRUE);
    duart_setup(DUART_ENABLE_RX_B,TRUE); /* enable rx & tx */

    duart_setup(DUART_RX_BAUD_A,17);    /* 9600 baud */
    duart_setup(DUART_TX_BAUD_A,17);    /* 9600 baud */
    duart_setup(DUART_RESET_ERROR_A,0);
    duart_setup(DUART_ENABLE_TX_A,FALSE);
    duart_setup(DUART_ENABLE_RX_A,FALSE); /* Reset error, disable rx & tx */
    duart_setup(DUART_RESET_TX_A,0);      /* Reset Tx */
    duart_setup(DUART_RESET_RX_A,0);      /* Reset Rx */
    duart_setup(DUART_DATA_BITS_A,8);
    duart_setup(DUART_STOP_BITS_A,1);
    duart_setup(DUART_PARITY_A,DUART_NONE);
    duart_setup(DUART_ENABLE_TX_A,TRUE);
    duart_setup(DUART_ENABLE_RX_A,TRUE); /* enable rx & tx */

  }
/*}}}*/
 
