/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : event.c
              Author : S.R.Christian, D. Johnston
   Date first issued : 12-Apr-91

Routines to support event handling on the IMS B016

Date        Modification                                    Initials
----        ------------                                    --------
02-Oct-91   Adapted from original                           SRC
18-Oct-91   Include return values from routines             SRC
******************************************************************/
/*}}}*/

#include <stdlib.h>
#include <process.h>
#include <misc.h>
#include "basic.h"
#include "event.h"

static void (*event_dispatch_table[256])();   /* Vector Table */
static unsigned char vector_used[256];
static void (*vector_table[256])();

int berr_happened; /* public variables */
int vme_pending;  /* copy of pending VME interrupt register */

void uninitialised_vector_routine(void)
/*{{{*/
{
}
/*}}}*/

int install_vector_handler(int vector,void (*routine)())
/*{{{*/
{
  if ((vector < 0) || (vector > 255))
    return(-1);
  event_dispatch_table[vector] = routine;
  vector_used[vector] = 1;
  return(0);
}
/*}}}*/

void standard_berr_handler()
/*{{{*/
{
  *(uint *)SPLAT_BERR = 0x100;
  berr_happened = 1;
}
/*}}}*/

void standard_int_ack_handler()
/*{{{*/
{
  vme_pending = *((uint*) VIC_VME_INT_STAT);
}
/*}}}*/

int install_berr_handler(void (*routine)())
/*{{{*/
{
  if (install_vector_handler(VECTOR_BERR,routine) < 0)
                    return(-1);

  berr_happened = 0;

  *(uint *)SPLAT_BERR = 0x100; 
  *(uint *) VIC_LOC_INT_VECT = VECTOR_BERR;
  *((uint *)VIC_LOC_INT_REG1)=0x13;  /* enable the local int,
                                               level sensitive, active low  */

  return(0);
}
/*}}}*/

int install_vme_int_ack_handler(void (*routine)())
/*{{{*/
{
  if (install_vector_handler(VECTOR_INTACK,routine) < 0)
    return(-1);

  vme_pending = 0;

  *(uint *) VIC_ERR_INT_VECT = VECTOR_INTACK;
  *(uint *) VIC_INT_CONT_ENB  = 0x03;

  return(0);
}
/*}}}*/

void int_handler(Process *p)
/*{{{*/
{
  int vector;
  void (*handler_routine)();
  p=p;
  for (;;)
    {
      ChanInInt(EVENT);      /* wait for an event */
      vector = *((uint *)VIC_IRQ_VECTOR);  /* get the vector */
      vector = (vector & 0xFF);

      handler_routine=event_dispatch_table[vector];
      (*handler_routine)();
      *((uint *)SPLAT_EV) = 0x100;  /* enable waiting for another event */
    }
}
/*}}}*/

int start_event_handler()
/*{{{*/
{
  int i;
  Process *p;

  for (i=0;i<256;i++) {                /* clear vector table */
        vector_table[i]= uninitialised_vector_routine;
        vector_used[i] = 0;
  }

  if ((p=ProcAlloc(int_handler,0,0))==NULL) {
    return(-1);
  }
  ProcRunHigh(p);

  
  /* install standard  handlers here */
  install_berr_handler(standard_berr_handler);
  install_vme_int_ack_handler(standard_int_ack_handler);
  
  return(0);
}
/*}}}*/

int ask_to_use_vector(int vector, void (*routine)())
/*{{{*/
{
  if ((vector < 0) || (vector > 255))
     return(-1);
  if (vector_used[vector] != 0)
     return(-1);
  vector_used[vector] = 1;
  install_vector_handler(vector,routine);
  return(0);
}
/*}}}*/

int give_next_vector(int vector, void (*routine)())
/*{{{*/
{
          if ((vector < 0) || (vector > 255))
                    return(-1);
          while (vector < 256)
            {
              if (ask_to_use_vector(vector++,routine) == 0)
              break;
            }
          if (vector < 256)
            {
              install_vector_handler(vector,routine);
              return(vector);
            }
  return(0);
}
/*}}}*/

int install_vme_int_handler(int level,int vector,void (*routine)())
/*{{{*/
{
  int temp;

  if ((level <1) || (level > 7))
    return(-1);
  if(install_vector_handler(vector,routine) < 0)
    return(-1);

  temp=(int)VIC_VME_INT_REG1;
  temp += ((level-1)*4);
  *((uint *)temp) = 0x03;

  return(0);
}
/*}}}*/

int install_duart_int_handler(void (*routine)())
/*{{{*/
{
  if (install_vector_handler(VECTOR_DUART,routine) < 0)
    return(-1);

  *(uint *) VIC_LOC_INT_VECT = VECTOR_DUART;
  *(uint *) VIC_LOC_INT_REG4 = 0x13;

  return(0);
}
/*}}}*/


