/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : vme.c
              Author : S.R.Christian, D. Johnston
   Date first issued : 12-Apr-91



Date        Modification                                    Initials
----        ------------                                    --------
02-Oct-91   Adapted from original                           SRC
18-Oct-91   Modified routine definitions                    SRC
******************************************************************/
/*}}}*/

#include "basic.h"
#include "vme.h"

/*{{{prog_slave_access*/
int prog_slave_access(void *decode_address,
                      int decode_window,
                      int address_type,
                      int writepost)
/* set up decode ram and control fields over a specified VME memory window */
{
  unsigned int decode_addr2;
  int temp,x,y;

  if (check_board_init() == 0)
    return (-1);
  if (decode_window <= 0)
    return(-1);
  
  if (address_type == 24) { 
    /* make the decode insensitive to the value of the top 8 address bits */
    decode_addr2 = (unsigned int)decode_address;    /* truncate to 24-bit address */
    decode_addr2 &= 0x00ff0000;
    decode_address =  (void *)decode_addr2;
    /* step across address window and set up decode ram for access */ 
    for (x = decode_window >> 16; x > 0 ; x--) { 
      for (y = 256; y > 0 ; y--) {   /* Write all possible upper address values */
        write_decode_ram(decode_address,2);
        decode_addr2=(unsigned int)decode_address;
        decode_addr2+=0x01000000;
        decode_address = (void *)decode_addr2;
      }
      decode_addr2=(unsigned int)decode_address;
      decode_addr2+=0x00010000;
      decode_address = (void *)decode_addr2;
    } 
  }
  else if (address_type == 32) { 
  /* step across address window and set up decode ram for access */
    for (x = decode_window >> 16; x > 0 ; x--) {
      write_decode_ram(decode_address,2);
      decode_addr2=(unsigned int)decode_address;
      decode_addr2+=65536;
      decode_address = (void *)decode_addr2;
    }                             
  } 
  else 
    return(-1); /* Don't allow anything other than A24 and A32 */

  temp = *(int *)VIC_SLAVE1_CON0 ; 
  temp &= 0x40;

  switch (address_type) {
    case 24: temp |= 0x04;
             break;
    case 32: temp |= 0x00;
             break;
    default: return(-1);
  }
  if (writepost == 1)
     temp |= 0x80;

  temp |= 0x10;  /* Set for this 32-bit board */

  /* set up control data */
  *(int *)VIC_SLAVE1_CON0 = temp;
  *(int *)VIC_SLAVE1_CON1 = 0x11;  /* Timing magic number */
  *(int *)VIC_LOC_TIME_REG = 0x73;
  *(int *)ENABLE_VME_SLAVE = 0x01;
  
  return(0);
}
/*}}}*/
/*{{{init_slot_one*/
int init_slot_one(int arbitration_style,int bus_timeout)
{
          int temp;

          if (check_board_init() == 0)
                    return (-1);
          if ((bus_timeout < 0) || (bus_timeout > 7))
                    return (-1);
          temp = *(int *)VIC_LOC_TIMER_REG;
          temp &= 0x1f;
          temp |= (bus_timeout << 5);
          *(int *)VIC_LOC_TIMER_REG = temp;
          temp = *(int *)VIC_VME_ARB_REQ ;
          temp &= 0x7f;
          if (arbitration_style == ARBITER_PRI)
                    temp |= 0x80;
          *(int *)VIC_VME_ARB_REQ = temp;
          return(0);
}
/*}}}*/
/*{{{init_vme_master*/
int init_vme_master(int request_level,int request_mode,int writepost)
{
  unsigned int temp;

  if (check_board_init() == 0)
     return (-1);
  if ((request_level < 0 ) || (request_level > 3) )
     return(-1);
  if ((request_mode < 0 ) || (request_mode > 3) )
     return(-1);

  /* assume this is a baord initialisation call */
  set_mux_0_transparent();

  temp = *(unsigned int *)VIC_SLAVE1_CON0;
  if (writepost)
     temp |= 0x40;
  else
     temp &= 0xbf;
  *(unsigned int *)VIC_SLAVE1_CON0 = temp;

  temp = *(unsigned int *)VIC_VME_ARB_REQ;
  temp &= 0x9f;
  temp |= (request_level << 5);
  *(unsigned int *)VIC_VME_ARB_REQ = temp;

  temp = *(unsigned int *)VIC_RELEASE_REG;
  temp &= 0x3f;
  temp |= (request_mode << 6);
  *(unsigned int *)VIC_RELEASE_REG = temp;
  
  return(0);
}
/*}}}*/
/*{{{turnon_vme_master*/
int turnon_vme_master()
{
  if (check_board_init() == 0)
    return (-1);
  *(int *)ENABLE_VME_MASTER = 0x01;

  return(0);
}
/*}}}*/
/*{{{turnoff_vme_master*/
int turnoff_vme_master(void)
{
  *(int *)ENABLE_VME_MASTER = 0x00;
  return(0);
}
/*}}}*/
/*{{{prog_master_access*/
int prog_master_access(void *address,
                       int address_window,
                       int data_type,
                       int address_type,
                       int lower_address,
                       int mux_fn)
/* This is a complete package for setting up master mode VME access over an
   address region */
{
  unsigned int addr2;
  int map_ram_entry,temp;
  unsigned int count;

  if (check_board_init() == 0)
    return (-1);
  if ((mux_fn > 7) || (mux_fn < 0))
    return(-1);
  if (address_window <= 0)
    return(-1);

  /* set up mapram entry bits */  
  map_ram_entry =  0;
  switch (data_type) {
      case 8:  map_ram_entry = DSIZ_8BIT;
               break;
      case 16: map_ram_entry = DSIZ_16BIT;
               break;
      case 32: map_ram_entry = DSIZ_32BIT;
               break;
      default: return(-1);
  }
  switch (address_type) {
      case 16: map_ram_entry |= ASIZ_16BIT;
               break;
      case 24: map_ram_entry |= ASIZ_24BIT;
               break;
      case 32: map_ram_entry |= ASIZ_32BIT;
               break;
      default: return(-1);
  }
   switch (lower_address) {
      case 3: map_ram_entry |= LADD11;
              break;
      case 2: map_ram_entry |= LADD10;
              break;
      case 1: map_ram_entry |= LADD01;
              break;
      case 0: map_ram_entry |= LADD00;
              break;
      default:return(-1);
  }
  /* always set master bit since thats what we asked for */
  map_ram_entry |= MASTER_PERMIT;
 
  /* make sure byte mux is disabled during reprogramming */ 
  turnoff_byte_mux();

  /* step across address window given setting all functions */
  for (count = ((unsigned int) address_window ) >> 20;
       count > 0; count--) {
    temp = shadow_read_map_ram(address) & 0x0070;
    temp |= map_ram_entry;
    shadow_write_map_ram(address,temp);
    program_mux_map_ram(mux_fn, address);
    addr2=(unsigned int)address;
    addr2+=0x00100000;
    address = (void *)addr2;
  }

  turnon_byte_mux();
  return(0);
}
/*}}}*/
/*{{{send_vme_interrupt*/
int send_vme_interrupt(int int_level)
{
  int temp;

  if (check_board_init() == 0)
    return (-1);

  if ( (int_level > 7 ) || ( int_level < 1) )
    return(-1);
  temp = (1<<int_level) | 0x01;
    *((unsigned int *)VIC_VME_INT_STAT) = temp;

  return(0);
}
/*}}}*/
/*{{{prog_vme_interrupt_vector*/
int prog_vme_interrupt_vector(int int_level,int vector)
{
  int temp;

  if (check_board_init() == 0)
    return (-1);
  if ( (int_level > 7 ) || ( int_level < 1) )
    return(-1);
  temp = VIC_VME_INT_VEC1 + ((int_level -1)*4);
  *((unsigned int *)temp) = vector;

  return(0);
}
/*}}}*/
