/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : moncoms.c
              Author : S.R.Christian, D. Johnston
   Date first issued : 12-Apr-91


File of common monitor command functions shared between versions of the cli

Date        Modification                                    Initials
----        ------------                                    --------
06-Oct-91   Created from basic routines                     SRC
******************************************************************/
/*}}}*/

/*{{{hard link declarations*/
static Channel* hard_links_in[] = {LINK0IN,LINK1IN,LINK2IN,LINK3IN};
static Channel* hard_links_out[] = {LINK0OUT,LINK1OUT,LINK2OUT,LINK3OUT};

/* default linkid to standard pipeline configuraton */
static long linkid = 2;
/*}}}*/

/*{{{display_memory*/
boolean display_memory(parse_t *pars_p, char *result_sym_p)
/* does an interactive formatted display of memory contents in numeric
   and ascii format */
{
  boolean   error;
  long      base;
  long      length;
  long      def_base;
  long      def_range;
  long      address;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_range, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) {
    tag_current_line(pars_p, "Invalid base address");
  }
  else {
    error = (cget_integer(pars_p, def_range, &length) || (length <= 0));
    if (error)
      tag_current_line(pars_p, "Invalid length");
    else {
      /* do a formatted display, address and four values a line.
         Watch for a bus error if we stray out of legal memory */
      long cnt;
      char  contents[17];
      int   i;
      
      address = base;
      cnt = 0;
      while ((cnt <= length) && !berr_flag) {
        print("#%8x: %8x %8x %8x %8x", address,
          *((long*)address),*(((long*)address)+1),
          *(((long*)address)+2),*(((long*)address)+3));
        for (i=0;i<16;i++) {
          contents[i] = *(((char*)address)+i);
          if (isprint(contents[i]) == 0)
            contents[i] = '.';
        }
        contents[16] = '\0';
        print(" (%s)\n", contents);
        /* protect ourselves from address rollover */
        if (address >= 0x7ffffff0)
          address = (long)0x80000000;
        else
          address += 16;
        cnt +=16;
      }
      if (berr_flag) {
        print("Bus error occurred on or near address #%8x\n", address);
        berr_flag = false;
        error = true;
      }
    }
  }
  return(error || assign_integer(result_sym_p, address, false));
}
/*}}}*/
/*{{{find_memory*/
boolean find_memory(parse_t *pars_p, char *result_sym_p)
/* does an search for specific memory contents */

{
  boolean   error;
  long      base;
  long      length;
  long      value;
  long      def_base;
  long      def_range;
  long      def_value;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_range, 10);
  evaluate_integer("DATAVALUE", &def_value, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) 
    tag_current_line(pars_p, "Invalid base address");
  else {
    error = (cget_integer(pars_p, def_range, &length) || (length <= 0));
    if (error)
      tag_current_line(pars_p, "Invalid length");
    else {
      error = cget_integer(pars_p, def_value, &value);
      if (error)
        tag_current_line(pars_p, "Invalid data value");
      else {
        /*{{{search and display*/
        /* do a search and display results if any*/
        long* address;
        int   found;
        long  cnt;
        
        cnt = 0;
        found = 0;
        address = (long*) base;
        while ((cnt < length) && !berr_flag) {
          if (*address == value){
            print("#%8x : #%8x (%ld)\n", (long)address, *address, *address);
            found++;
          }
          cnt++;
          address = (long*)(base+cnt);
        }
        if (berr_flag) {
          print("Bus error occurred on or near address #%8x\n", address);
          berr_flag = false;
          error = true;
        }
        if (found > 0)
          print("\n%d matches in %ld bytes\n", found, length);
        /*}}}*/
      }
    }
  }
  return(error || assign_integer(result_sym_p, base, false));
}
/*}}}*/
/*{{{fill_memory*/
boolean fill_memory(parse_t *pars_p, char *result_sym_p)
/* writes specific memory contents */

{
  boolean   error;
  long      base;
  long      length;
  long      value;
  long      def_base;
  long      def_range;
  long      def_value;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_range, 10);
  evaluate_integer("DATAVALUE", &def_value, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) 
    tag_current_line(pars_p, "Invalid base address");
  else {
    error = (cget_integer(pars_p, def_range, &length) || (length <= 0));
    if (error)
      tag_current_line(pars_p, "Invalid length");
    else {
      error = cget_integer(pars_p, def_value, &value);
      if (error)
        tag_current_line(pars_p, "Invalid data value");
      else {
        /*{{{fill words*/
        /* do a word oriented global fill operation*/
        long* address;
        int  cnt;
        
        cnt = 0;
        address = (long*) base;
        while (((long)address < (base + length)) && !berr_flag) {
          *address = value;
          address++;
        }
        if (berr_flag) {
          print("Bus error occurred on or near address #%8x\n", address);
          berr_flag = false;
          error = true;
        }
        
        /*}}}*/
      }
    }
  }
  return(error || assign_integer(result_sym_p, base, false));
}
/*}}}*/
/*{{{copy_memory*/
boolean copy_memory(parse_t *pars_p, char *result_sym_p)
/* copies memory contents to new address range*/

{
  boolean   error;
  long      base;
  long      length;
  long      newbase;
  long      def_base;
  long      def_range;
  long      def_newbase;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_range, 10);
  evaluate_integer("ADDRESSVALUE", &def_newbase, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) 
    tag_current_line(pars_p, "Invalid base address");
  else {
    error = (cget_integer(pars_p, def_range, &length) || (length <= 0));
    if (error)
      tag_current_line(pars_p, "Invalid length");
    else {
      error = (cget_integer(pars_p, def_newbase, &newbase) ||
               (newbase == base));
      if (error)
        tag_current_line(pars_p, "Invalid new base address");
      else {
        /*{{{move memory*/
        /* do a memory copy operation using routine that works in overlapped case*/
        
        memmove((char*)newbase, (char*)base, (int) length);
        if (berr_flag) {
          print("Bus error occurred during copy! \n");
          berr_flag = false;
          error = true;
        }
        
        /*}}}*/
      }
    }
  }
  return(error || assign_integer(result_sym_p, base, false));
}
/*}}}*/
/*{{{mapram*/
boolean mapram(parse_t *pars_p, char *result_sym_p)
/* does an formatted display of mapram memory contents in numeric
   and ascii format */
{
  boolean   error;
  long      base;
  long      def_base;
  long      mapram_entry;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) {
    tag_current_line(pars_p, "Invalid base address");
  }
  else {
    /*{{{display data*/
    /* do a formatted display, decoding values as required */
      
    mapram_entry = shadow_read_map_ram((void*)base);
    print("#%8x: %x (", base, mapram_entry);
    if ((mapram_entry & MASTER_PERMIT) > 0) print("MASTER ");
    switch(mapram_entry & 0x6000) {
      case AM_UDATA: print("UDATA "); break;
      case AM_UPROG: print("UPROG "); break;
      case AM_SDATA: print("SDATA "); break;
      case AM_SPROG: print("SPROG "); break;
    }
    switch(mapram_entry & 0x180) {
      case ASIZ_AMSOURCE: print("AMSOURCE "); break;
      case ASIZ_32BIT: print("A32BIT "); break;
      case ASIZ_24BIT: print("A24BIT "); break;
      case ASIZ_16BIT: print("A16BIT "); break;
    }
    switch(mapram_entry & 0x1800) {
      case LADD00: print("LADD00 "); break;
      case LADD01: print("LADD01 "); break;
      case LADD10: print("LADD10 "); break;
      case LADD11: print("LADD11 "); break;
    }
    switch(mapram_entry & 0x600) {
      case DSIZ_32BIT: print("D32BIT "); break;
      case DSIZ_16BIT: print("D16BIT "); break;
      case DSIZ_8BIT: print("D8BIT "); break;
      case DSIZ_THREE: print("DTHREE "); break;
    }
    print("bytemux fn %x)\n", ((mapram_entry & 0x70)>>4));
    /*}}}*/
  }
  return(error || assign_integer(result_sym_p, mapram_entry, false));
}
/*}}}*/
/*{{{set_mux*/
boolean set_mux(parse_t *pars_p, char *result_sym_p)
/* sets mapram and mux contents for address range */
{
  boolean   error;
  long      base;
  long      length;
  long      def_base;
  long      def_length;
  long      def_entry;
  long      mapram_entry;
  long      read_swap;
  long      write_swap;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_length, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) 
    tag_current_line(pars_p, "invalid base address");
  else {
    def_entry = shadow_read_map_ram((void*)base);
    error = cget_integer(pars_p, def_length, &length);
    if (error)
      tag_current_line(pars_p, "invalid address range");
    else {
      error = cget_integer(pars_p, def_entry, &mapram_entry);
      if (error)
        tag_current_line(pars_p, "invalid mapram entry");
      else {
        error = cget_integer(pars_p, 0x76543210, &read_swap);
        if (error)
          tag_current_line(pars_p, "invalid readswap code");
        else {
          error = cget_integer(pars_p, 0x76543210, &write_swap);
          if (error)
            tag_current_line(pars_p, "invalid writeswap code");
          else {
            /*{{{program data into mapram and mux*/
            /* program matching mux and mapram entries for
               each 1 Mb page of memory space in wanted range*/
            int   which_swap;
            char* page;
            
            turnoff_byte_mux();
            which_swap = (int)((mapram_entry & 0x70)>>4);
            page = (char*)((unsigned)base & 0xfff00000);              
            print("Programming mapram and mux function %d from address #%x\n", which_swap, (int)page);
            program_byte_mux(which_swap, (int)read_swap, (int) write_swap);
            while ((long)page < (base+length)) {
              shadow_write_map_ram((void*)page, (int)mapram_entry);
              program_mux_map_ram(which_swap, (void*)page);
              page += 0x100000;
            }
            turnon_byte_mux();
            /*}}}*/
          }
        }
      }
    }
  }
  return(error || assign_integer(result_sym_p, mapram_entry, false));
}
/*}}}*/
/*{{{setup_slave*/
boolean setup_slave(parse_t *pars_p, char *result_sym_p)
/* sets decode logic for slave access to board memory */

{
  boolean   error;
  long      base;
  long      length;
  long      add_value;
  long      def_base;
  long      def_range;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_range, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) 
    tag_current_line(pars_p, "expected slave base address");
  else {
    error = (cget_integer(pars_p, def_range, &length) || (length <= 0));
    if (error)
      tag_current_line(pars_p, "invalid length");
    else {
      error = (cget_integer(pars_p, 32, &add_value) ||
               ((add_value != 32) && (add_value != 24) && (add_value !=16)));
      if (error)
        tag_current_line(pars_p, "invalid address width value");
      else {
        /* set up slave access on this region */
        prog_slave_access((void*)base, (int)length, (int)add_value, false);
      }
    }
  }
  return(error || assign_integer(result_sym_p, base, false));
}
/*}}}*/
/*{{{write_duart*/
boolean write_duart(parse_t *pars_p, char *result_sym_p)
/*  */
{
  boolean   error;
  char      string[max_tok_len];
  
  error = (cget_string(pars_p, "", string, max_tok_len) ||
           (strlen(string) == 0));
  if (error)
    tag_current_line(pars_p, "expected output string");
  else {
    duart_write_b(string);
    duart_write_b("\n");
  }
  return(error || assign_string(result_sym_p, string, false));
}
/*}}}*/
/*{{{read_duart*/
boolean read_duart(parse_t *pars_p, char *result_sym_p)
/* Reads input string from the duart port */
{
  boolean   error;
  char      string[max_tok_len];
  char      c;
  short     cnt;
  boolean   eol;
  
  error = false;
  cnt = 0;
  eol = false;
  string[0] = '\0';
  c=' ';

  /* read characters one by one and perform line editing functions before
     putting them in the read buffer. Characters are echoed as accepted. */
  while((c != '\0') && !eol && (cnt < max_tok_len)) {
    c = duart_getchar_b();
    if (isprint(c)) {
      string[cnt] = c;
      cnt++;
      duart_putchar_b(c);
    }
    else switch(c) {
      /* control characters are recognised and dealt with. Escape
         is used to repeat the previous line of input */
      case '\r' :
      case '\n' : string[cnt++] = '\0';
                  duart_putchar_b('\r');
                  duart_putchar_b('\n');
                  eol = true;
                  strcpy(last_command, string);
                  break;
      case '\b' : if (cnt > 0) {
                    cnt--;
                    duart_putchar_b('\b');
                    duart_putchar_b(' ');
                    duart_putchar_b('\b');
                  }
                  break;
      case 0x1b : strcpy(string, last_command);
                  duart_putchar_b('\r');
                  duart_putchar_b('\n');
                  duart_write_b(string);
                  cnt = strlen(string);
                  break;
    }
  }
  if (c == '\0') error = true;
  return(error || assign_string(result_sym_p, string, false));
}
/*}}}*/
/*{{{read_rtc*/
boolean read_rtc(parse_t *pars_p, char *result_sym_p)
/* can set a new time value into the rtc and/or display result */
{
  boolean   error;
  time_t    time_code;
  
  error = cget_integer(pars_p, -1, (long*)&time_code);
  if (error)
    tag_current_line(pars_p, "expected new time value");
  else {
    if (((long)time_code) >= 0) {
      /* program new code given into the rtc */
      print("Setting new time: %s\n", ctime(&time_code));
      rtc_settimeofday(&time_code);
    }
    else {
      /* read time as held by the clock and format to display */
      rtc_gettimeofday(&time_code);
      if((long)time_code <= 0)
        print("Odd value %d from RTC - is standby power on?\n", (long)time_code);
      else
        print("RTC clock: %s\n", ctime(&time_code));
    }
  }
  return(error || assign_integer(result_sym_p, (long)time_code, false));
}
/*}}}*/
/*{{{read_rom_status*/
boolean read_rom_status(parse_t *pars_p, char *result_sym_p)
/* verify Flash ROM status */
{
  boolean   error = false;
  int       size;
  int       data;
  int       code;
  int       i;

  reset_rom();
  data = blank_check_rom();

  turnon_vpp();
  size = find_rom_size();
  print("Flash ROM size #%x - data state: ", size);
  switch(data) {
    case 0xff: print("blank (#ff)\n"); break;
    case 0x00: print("zero (#00)\n"); break;
    case -1  : print("programmed\n"); break;
    default  : print("unknown\n"); break;
  }
  for (i=0;i<=3;i++) {
    code = read_rom_device_code(i);
    print("device %d: code %x: ", i, code);
  }
  print("\n");
  if (code == 0xff)
    print("Check VPP is enabled!\n");
  reset_rom();
  return(error || assign_integer(result_sym_p, size, false));
}
/*}}}*/
/*{{{reset_ss*/
boolean reset_ss(parse_t *pars_p, char *result_sym_p)
/*  */
{
  boolean   error;
  long      anal_flag;
  long      tx_err;
  long*     addr;
  
  error = cget_integer(pars_p, false, &anal_flag);
  if (error) {
    tag_current_line(pars_p, "expected analyse flag");
  }
  else {
    error = (cget_integer(pars_p, linkid, &linkid) 
             || (linkid < 0) || (linkid > 3));
    if (error) {
      tag_current_line(pars_p, "expected link number");
    }
    else {
      /*{{{do reset/analyse cycle*/
      if (anal_flag) {
        addr = (long*)TX_ANALYSE_REG; 
        *addr = 1;
        ProcWait(2000);
      }
      addr = (long*)TX_RESET_REG; 
      *addr = 1;
      ProcWait(2000);
      if (anal_flag) {
        addr = (long*)TX_ANALYSE_REG;
        *addr = 0;
        ProcWait(2000);
      }
      addr = (long*)TX_RESET_REG;
      *addr = 0;
      ProcWait(2000);
      addr = (long*)TX_ERROR_REG;
      tx_err = *addr ^ 0x1; /* reads the wrong way up! */
      print("error status = %d (linkid=%d)\n", tx_err, linkid);
      /*}}}*/
    }
  }
  return(error || assign_integer(result_sym_p, tx_err, false));
}
/*}}}*/
/*{{{peek_ss*/
boolean peek_ss(parse_t *pars_p, char *result_sym_p)
/* does an interactive formatted display of memory contents in numeric
   and ascii format - memory contents obtained from remote transputer
   via peek instructions */
{
  boolean   error;
  long      base;
  long      length;
  long      def_base;
  long      def_range;
  long      address;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_range, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) {
    tag_current_line(pars_p, "expected base address");
  }
  else {
    error = (cget_integer(pars_p, def_range, &length) || (length <= 0));
    if (error)
      tag_current_line(pars_p, "expected length");
    else {
      /*{{{obtain and display data*/
      /* Obtain each word by peekin an assumed reset processor down
         the defined link.
         do a formatted display, address and four values a line. */
      long     cnt;
      char     contents[17];
      int      i;
      boolean  time_out;
      Channel* op;
      Channel* ip;
      long     values[4];
      long     peek = 1;
      long     temp;
      int      time;
      
      op = hard_links_out[linkid];
      ip = hard_links_in[linkid];
      time_out = false;
      address = base;
      cnt = 0;
      while ((cnt <= length) && !time_out) {
        for (i=0; (i<4) && !time_out; i++) {
          temp=address+(4*(long)i);
          time = ProcTime();
          time = ProcTimePlus(time, 10000);    
          time_out = ChanOutTimeFail(op, &peek, 1, time);
          time_out = ChanOutTimeFail(op, &temp, 4, time); 
          time_out = ChanInTimeFail(ip, &values[i], 4, time);
        } 
        if (!time_out) {
          print("#%8x: %8x %8x %8x %8x", address,
            values[0], values[1], values[2], values[3]);
          for (i=0;i<16;i++) {
            contents[i] = *(((char*)&values)+i);
            if (isprint(contents[i]) == 0)
              contents[i] = '.';
          }
          contents[16] = '\0';
          print(" (%s)\n", contents);
          /* protect ourselves from address rollover */
          if (address >= 0x7ffffff0)
            address = (long)0x80000000;
          else
            address += 16;
          cnt +=16;
        }
      }
      if (time_out) {
        print("Link time out at or near address #%8x\n", address);
        error = true;
      }
      /*}}}*/
    }
  }
  return(error || assign_integer(result_sym_p, address, false));
}
/*}}}*/
/*{{{poke_ss*/
boolean poke_ss(parse_t *pars_p, char *result_sym_p)
/* Fills memory of external transputer with data value by poke operations */
{
  boolean   error;
  long      base;
  long      length;
  long      value;
  long      def_base;
  long      def_range;
  long      def_value;

  /* get default values from symbol variables */
  evaluate_integer("BASEADDRESS", &def_base, 10);
  evaluate_integer("RANGE", &def_range, 10);
  evaluate_integer("DATAVALUE", &def_value, 10);
    
  error = cget_integer(pars_p, def_base, &base);
  if (error) 
    tag_current_line(pars_p, "Invalid base address");
  else {
    error = (cget_integer(pars_p, def_range, &length) || (length <= 0));
    if (error)
      tag_current_line(pars_p, "Invalid length");
    else {
      error = cget_integer(pars_p, def_value, &value);
      if (error)
        tag_current_line(pars_p, "Invalid data value");
      else {
        /*{{{Fill remote memory*/
        /* Fill each word by poking a value to the defined link. */
        long     cnt;
        boolean  time_out;
        Channel* op;
        long     poke = 0;
        long     address;
        int      time;
        
        op = hard_links_out[linkid];
        time_out = false;
        address = base;
        cnt = 0;
        while ((cnt <= length) && !time_out) {
          time = ProcTime();
          time = ProcTimePlus(time, 10000);
          time_out = ChanOutTimeFail(op,&poke, 1, time); 
          time_out = ChanOutTimeFail(op,&address, 4, time);
          time_out = ChanOutTimeFail(op,&value, 4, time);
        
          /* protect ourselves from address rollover */
          if (address >= 0x7ffffffc)
            address = (long)0x80000000;
          else
            address += 4;
          cnt+=4;
        }
        if (time_out) {
          print("Link time out at or near address #%8x\n", address);
          error = true;
        }
        /*}}}*/
      }
    }
  }
  return(error || assign_integer(result_sym_p, base, false));
}
/*}}}*/


