/*{{{ COMMENT Standard Header*/
/******************************************************************
COPYRIGHT (C) INMOS LIMITED - 1990, 1991

    Source file name : monitor.c
              Author : S.R.Christian, D. Johnston
   Date first issued : 2-Aug-90

Contains main routines for a host based monitor exercising the IMS B016

Date        Modification                                    Initials
----        ------------                                    --------
02-Oct-91   Adapted from original monitor.c by DJ           SRC
******************************************************************/
/*}}}*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <iocntrl.h>
#include <process.h>
#include <channel.h>
#include <misc.h>
#include <time.h>
#include "vme.h"
#include "duart.h"
#include "event.h"
#include "basic.h"
#include "rtc.h"
#include "rom.h"
#include "clilib.h"

/*{{{console io routines for hosted environment*/
int io_write(char* buffer)
/* support for 'console' string output in a host supported environment */
{
   write(1, buffer, strlen(buffer));
   return(0);
}

int io_read(char* prompt, char* buffer,long buflen)
/* support for a read of a null terminated input line from a 'console' */
{
  int temp;
  int cnt;

  print(prompt); 
  fflush(stdin);

  cnt = 0;
  temp = 0; 
  buffer[cnt] = getc(stdin);
  while ((cnt < buflen) && (buffer[cnt] != '\n') && 
         ((temp = getc(stdin)) != EOF))
     buffer[++cnt] = temp;
  
  buffer[cnt] = '\0';
  return(cnt);
}
/*}}}*/

/*{{{static variables*/
static char last_command[max_tok_len];
static boolean master_mode_set=FALSE;
static boolean slot_one_set=FALSE;
static berr_flag=FALSE;


/*}}}*/

/*{{{void io_setup(void);*/
void io_setup(void)
  {

    duart_setup(DUART_RX_BAUD_B,BAUD_9600);
    duart_setup(DUART_TX_BAUD_B,BAUD_9600);
    duart_setup(DUART_RESET_ERROR_B,0);
    duart_setup(DUART_ENABLE_TX_B,FALSE);
    duart_setup(DUART_ENABLE_RX_B,FALSE); /* RESET ERROR, DISABLE RX & TX */
    duart_setup(DUART_RESET_TX_B,0);   /* RESET tX */
    duart_setup(DUART_RESET_RX_B,0);   /* RESET rX */
    duart_setup(DUART_DATA_BITS_B,8);
    duart_setup(DUART_STOP_BITS_B,1);
    duart_setup(DUART_PARITY_B,DUART_NONE);
    duart_setup(DUART_ENABLE_TX_B,TRUE);
    duart_setup(DUART_ENABLE_RX_B,TRUE); /* ENABLE RX & TX */

    duart_setup(DUART_RX_BAUD_A,BAUD_9600);
    duart_setup(DUART_TX_BAUD_A,BAUD_9600);
    duart_setup(DUART_RESET_ERROR_A,0);
    duart_setup(DUART_ENABLE_TX_A,FALSE);
    duart_setup(DUART_ENABLE_RX_A,FALSE); /* RESET ERROR, DISABLE RX & TX */
    duart_setup(DUART_RESET_TX_A,0);      /* RESET tX */
    duart_setup(DUART_RESET_RX_A,0);      /* RESET rX */
    duart_setup(DUART_DATA_BITS_A,8);
    duart_setup(DUART_STOP_BITS_A,1);
    duart_setup(DUART_PARITY_A,DUART_NONE);
    duart_setup(DUART_ENABLE_TX_A,TRUE);
    duart_setup(DUART_ENABLE_RX_A,TRUE); /* ENABLE RX & TX */

  }
/*}}}*/
/*{{{int is_safe_access(int address);*/
int is_safe_access(int address)
  {
  int megabyte;
  int outside;
  int master_bit;
    master_bit = (shadow_read_map_ram((void *)address) & 0x8000) >> 15;
    if (master_mode_set==FALSE) {master_bit=FALSE;}
    megabyte=(address & 0xFFF00000) >> 20;
    if ((megabyte >= 0x0800) && (megabyte <= 0x0803))
      {outside=FALSE;}
    else
      {outside=TRUE;}
    if ((outside==TRUE) && (master_bit == 0)) {return(FALSE);}
      else
        {return(TRUE);}

  }
/*}}}*/
/*{{{void my_berr_handler(void);*/
void my_berr_handler(void)
{
    /* *(unsigned int *)SPLAT_BERR=0x01; old code in error? */
    *(unsigned int *)SPLAT_BERR=0x100;
    berr_flag=TRUE;
}
/*}}}*/

#include "moncoms.c"

/*{{{prog_rom*/
boolean prog_rom(parse_t *pars_p, char *result_sym_p)
/* Programs data file into Flash ROM devices */
{
  boolean   error;

  char   filename[FILENAME_MAX];
  FILE*  fp;
  int    c;
  int    romsize;
  int    filesize;
  int    val;

  error = (cget_string(pars_p, "romdata.btl", filename, FILENAME_MAX) ||
           (strlen(filename) == 0));
  if (error)
    tag_current_line(pars_p, "Invalid binary filename");
  else {
    if ((fp=fopen(filename, "rb")) == NULL) {
      print("Can't open %s for reading\n",filename);
      error = true;
    }
    else {
      /*{{{erase and programming*/
      /* read the file & program the roms */
      int    address;
      int    which_rom;
      
      fseek(fp, 0, SEEK_END);
      filesize = (int) ftell(fp);
      rewind(fp);
      print("Opened binary input file \"%s\" of size #%x\n",filename, filesize);
          
      reset_rom();
      turnon_vpp();
      romsize=find_rom_size();
      reset_rom();
      print("Board ROM size #%x\nErasing ", romsize);
      /*{{{Erase ROM*/
      turnon_vpp();
      print(".");
      val=erase_rom(0);
      print(".");
      val=erase_rom(1);
      print(".");
      val=erase_rom(2);
      print(".");
      val=erase_rom(3);
      print(".");
      reset_rom();
      /*}}}*/
      val = blank_check_rom(); 
      if (val != 0xff) {
        print(" Erase check failed at #%x\n", val);
        error = true;
      }
      else {
        print(" Done\n");
      
        reset_rom();
        turnon_vpp();
        print("Programming data ");
      
        address = (romsize - filesize);
        c = (char) fgetc(fp);
        while ((c != EOF) && !error && (address < romsize)) {
          which_rom = address & 0x3;
          val = program_byte(which_rom,address,c);
          if (val == -1) {
            print("\nFailed to program byte #%x in ROM %d\n", address, which_rom);
            error = true;
          }
          if ((address % 0x1000) == 0) {
            print(".");
          }
          address++;
          c = (char) fgetc(fp);
        }
        if (!error) print(" Done\n");
      }
      /*}}}*/
    }
    fclose(fp);
    turnoff_vpp();
    reset_rom();
  }
  return(error || assign_integer(result_sym_p, filesize, false));
}
/*}}}*/

/*{{{command setup*/
static boolean command_setup()
/* setup routine for IMS B016 monitor commands */
{

  register_command("DISPLAY", display_memory, 
                   "<address><range> Displays memory contents");
  register_command("SEARCH", find_memory, 
                   "<address><range><value> Searchs for memory contents");
  register_command("FILL", fill_memory, 
                   "<address><range><value> Fill memory with value");
  register_command("COPY", copy_memory, 
                   "<address><range><newaddress> Copies memory contents");
  register_command("MAPRAM", mapram, 
                   "<address> Display of mapram entry");
  register_command("MUXRAM", set_mux, 
                   "<address><range><mapcode><rswop><wswop> Set up mux & mapram entry");
  register_command("SLAVE", setup_slave, 
                   "<address><range><addwidth> Set up slave access region");
  register_command("ROM", read_rom_status, 
                   "Display of ROM status");
  register_command("FLASH", prog_rom, 
                   "<filename> Programs Flash ROMs");
  register_command("BWRITE", write_duart, 
                   "<string> Writes to duart port B");
  register_command("BREAD",  read_duart, 
                   "Reads string from duart port B");
  register_command("RTC",  read_rtc, 
                   "<timeval> Sets and/or reads RTC registers");
  register_command("INIT",  reset_ss, 
                   "<analflag><linkid> Reset/analyse on subsystem port");
  register_command("PEEK",  peek_ss, 
                   "<address><range> Displays remote memory");
  register_command("POKE",  poke_ss, 
                   "<address><range><value> Sets remote memory");

  /* utility default variables */
  assign_integer("MEMSIZE", 0x400000, true);
  assign_integer("BASEADDRESS", 0x80000000, false);
  assign_integer("RANGE", 0x100, false);
  assign_integer("DATAVALUE", 0, false);
  assign_integer("ADDRESSVALUE", 0x80000000, false);

  /* H/W predefines for mapram */

  assign_integer("MASTER", MASTER_PERMIT, true);
  assign_integer("D8BIT", DSIZ_8BIT, true);
  assign_integer("D16BIT", DSIZ_16BIT, true);
  assign_integer("D32BIT", DSIZ_32BIT, true);
  assign_integer("A32BIT", ASIZ_32BIT, true);
  assign_integer("A24BIT", ASIZ_24BIT, true);
  assign_integer("A16BIT", ASIZ_16BIT, true);
  assign_integer("LADD00", LADD00, true);
  assign_integer("LADD01", LADD01, true);
  assign_integer("LADD10", LADD10, true);
  assign_integer("LADD11", LADD11, true);
  /* MUX functions */
  assign_integer("FN1", 0x10, true);
  assign_integer("FN2", 0x20, true);
  assign_integer("FN3", 0x30, true);
  assign_integer("FN4", 0x40, true);
  assign_integer("FN5", 0x50, true);
  assign_integer("FN6", 0x50, true);
  assign_integer("FN7", 0x70, true);



  return(false);
}
/*}}}*/
/*{{{main*/
int main(int argc, char** argv)
{
   char* startup_file;

   /* do board level initialisation */

   basic_board_init();
   start_event_handler();
   install_berr_handler(my_berr_handler);
   
   /* initialise on board io */
   duart_init();  
   io_setup();    
   duart_use_xon_a(TRUE);
   duart_use_xon_b(TRUE);

   /* assert master role */
   init_slot_one(ARBITER_PRI,VMETIMEOUT_64US);
   init_vme_master(3, RELMODE_BCAP, false);
   set_mux_0_transparent();
 
   slot_one_set=TRUE;

   /* do we have a startup command file? */
   if (argc > 0)
     startup_file = argv[1];
   else
     startup_file = NULL;

   /* initialise buffer for command editing support */
   strcpy(last_command,"help");

   /* print title and start interactive loop */
   print("\nIMS B016 monitor v1.0\n\n");

   cli_main(command_setup, 1000, 16, "B016> ", startup_file);
  
   exit_terminate(0);
}
/*}}}*/

