/******************************************************************************
* This is a totally fabricated, completely useless test program to test
* out the profiler. All it does is generate some matrixes, and then
* passes them down the pipeline as a re-circulating token. Hence at any
* one time, only one processor should be busy, and we should see a
* nice sort of step  arrangement viz:
*
* Processor 1 : ***************
* Processor 2 :               *************
* Processor 3 :                            **************
* 
* 
* Also, this example is constructed processors do not start executing
* user code at the same time. The profiler has to account for this by
* magically munging results to synchronize the clocks. The profiler
* reveals some interesting results. For example, if you run on a mixed
* network of T4 and T8's the length of the time to compute the packet
* varies wildly between the two because the T4 does not have an FPU. It
* also reveals the effect of having slower external memory on otherwise
* identical processors. I thought it was a bug!! Also note that when
* using this program, the profiler will show that the root processor, ie
* the one running this code does not have a smooth overlap when the
* packet is pushed out like all the others. This is because it is doing
* printf's to the host which the profiler picks up.
* 
* The pipeline elements signal a high priority thread after they have
* finished their compute phase, which then waits and then sits in a busy
* loop. This is to test that syncing between processors works with high
* priority processes present.
*/
#include <stdio.h>
#include <stdlib.h>
#include <channel.h>
#include <misc.h>
#include <math.h>
#include <process.h>
#include "matrix.h"


static Channel *toPipeline;
static Channel *fromPipeline;


/*
 * Generates a random matrix with elements in the range 0 up to 'mult'
 */

void GenerateRandomMatrix(Matrix matrix,double mult)
{
 int row,col;

 for(row=0;row<MATRIX_SIZE;row++) {
     for(col=0;col<MATRIX_SIZE;col++) {
         matrix[row][col]=(( (double) rand() ) / (double)RAND_MAX) *mult;
      }
 }
 
}



/* 
 * Multiplies two matrixes together, and bungs them in results
 */
 
void MatrixMul(Matrix matrix1,Matrix matrix2,Matrix result)
{
 int row,col,x,z;
 double sum;
 int start;

 for(row=0;row<MATRIX_SIZE;row++) {
     for(col=0;col<MATRIX_SIZE;col++) {
         sum=0.0;
         for(x=0;x<MATRIX_SIZE;x++) {
             sum+=matrix1[row][x]*matrix2[x][col];
         }
         result[row][col]=sum;
     }     
 }

}


/*
 * Prints out a matrix
 */
void PrintMatrix(Matrix m)
{
 int row,col;

 for(row=0;row<MATRIX_SIZE;row++) {
     for(col=0;col<MATRIX_SIZE;col++) {
         printf("%lg ",m[row][col]);
      }
      printf("\n");
 }
 printf("\n");

} 


#define NUMBER_TO_COUNT 25
#define NUMBER_MATRIX_MUL 25

int main()
{
 Matrix m1,m2,result;
 int x;
 int start;
 int start1; 
 int counts;
 int num;
 Channel *sync;
 Process *p;
 
 fromPipeline=(Channel*) get_param(3); 
 toPipeline=(Channel*) get_param(4);
 printf("Matrix size is %d\n",sizeof(Matrix));
 /* Do the loop 3 times */
 for(num=0;num<3;num++) {
     GenerateRandomMatrix(m1,1.0);
     GenerateRandomMatrix(m2,100.0);
     printf("Matrix is :-\n");
     PrintMatrix(m1);
     start=ProcTime();
     ChanOut(toPipeline,m2,sizeof(Matrix));
     printf("Transfer time is %g seconds\n",(double)ProcTimeMinus(ProcTime(),start)/15625);
     start=ProcTime();     
     for(counts=0;counts<NUMBER_TO_COUNT;counts++) {         
         ChanIn(fromPipeline,m1,sizeof(Matrix));
         start1=ProcTime();
         for(x=0;x<NUMBER_MATRIX_MUL;x++) {
             MatrixMul(m1,m2,result);
         }
         ChanOut(toPipeline,m1,sizeof(Matrix));
         printf("Loop time is %g seconds\n",(double)ProcTimeMinus(ProcTime(),start1)/15625);
     }     
     ChanIn(fromPipeline,m1,sizeof(Matrix));     /* Sink packet */
 }
 
 exit_terminate(EXIT_SUCCESS);
}
