/*------------------------------------------------------------------------------
--
-- File    : metsetup.c
--   metalinkio setup module
--
-- Copyright 1992 INMOS Limited
--
-- Date    : 6/4/92
-- Version : 1.3
--
-- 29/06/92 - dave edwards; based on parts of metlnkio.c, lmethod.c
--
------------------------------------------------------------------------------*/

#include <stdio.h>
#include "metsetup.h"
#include "ioptype.h"

static int Undefined ()
{
  fprintf (stderr, "***FATAL ERROR : metsetup.c -\nundefined routine\n");
  fflush (stderr);
  return (-1);
}

/****
 **** state variables maintained by this module
****/
int     ms_Maxmethodval = MS_MAXLMETHODS;
int     ms_Available[MS_MAXLMETHODS];
t_IOFnptr ms_Openlink[MS_MAXLMETHODS];
t_IOFnptr ms_Closelink[MS_MAXLMETHODS];
t_IOFnptr ms_Readlink[MS_MAXLMETHODS];
t_IOFnptr ms_Writelink[MS_MAXLMETHODS];
t_IOFnptr ms_Resetlink[MS_MAXLMETHODS];
t_IOFnptr ms_Analyselink[MS_MAXLMETHODS];
t_IOFnptr ms_TestError[MS_MAXLMETHODS];

/****
 **** strings used to define methods found in connection database
 **** file. must be same as values defined in header file
****/
/* define strings found in connection database file */
char *ms_Methodnames[] = {
  "undefined",
  "b004",
  "b008",
  "b010",
  "b011",
  "b014",
  "drx11",
  "qt0",
  "b015",
  "ibm_cat",
  "b016",
  "udp",
  "tcp",
  "b017"
};


void ms_Init ()
{
  int i;

  /* initialise them all as undefined */
  for (i = 0; i < MS_MAXLMETHODS; i++) {
    ms_Available[i] = 0;
    ms_Openlink[i] = Undefined;
    ms_Closelink[i] = Undefined;
    ms_Readlink[i] = Undefined;
    ms_Writelink[i] = Undefined;
    ms_Resetlink[i] = Undefined;
    ms_Analyselink[i] = Undefined;
    ms_TestError[i] = Undefined;
  }
    
      
#ifdef LNKtcp
  ms_Available[TCP_LINKOPS] = 1;
  ms_Openlink[TCP_LINKOPS]     = TCPOpenLink;
  ms_Closelink[TCP_LINKOPS]    = TCPCloseLink;
  ms_Readlink[TCP_LINKOPS]     = TCPReadLink;
  ms_Writelink[TCP_LINKOPS]    = TCPWriteLink;
  ms_Resetlink[TCP_LINKOPS]    = TCPResetLink;
  ms_Analyselink[TCP_LINKOPS]  = TCPAnalyseLink;
  ms_TestError[TCP_LINKOPS]    = TCPTestError;
#endif

#ifdef SUN4
#ifdef LNKb011
  ms_Available[B011] = 1;
  ms_Openlink[B011]    = B011OpenLink;
  ms_Closelink[B011]   = B011CloseLink;
  ms_Readlink[B011]    = B011ReadLink;
  ms_Writelink[B011]   = B011WriteLink;
  ms_Resetlink[B011]   = B011ResetLink;
  ms_Analyselink[B011] = B011AnalyseLink;
  ms_TestError[B011]   = B011TestError;
#endif

#ifdef LNKb014
  ms_Available[B014] = 1;
  ms_Openlink[B014]    = B014OpenLink;
  ms_Closelink[B014]   = B014CloseLink;
  ms_Readlink[B014]    = B014ReadLink;
  ms_Writelink[B014]   = B014WriteLink;
  ms_Resetlink[B014]   = B014ResetLink;
  ms_Analyselink[B014] = B014AnalyseLink;
  ms_TestError[B014]   = B014TestError;
#endif
    
#ifdef LNKb016
  ms_Available[B016] = 1;
  ms_Openlink[B016]    = B016OpenLink;
  ms_Closelink[B016]   = B016CloseLink;
  ms_Readlink[B016]    = B016ReadLink;
  ms_Writelink[B016]   = B016WriteLink;
  ms_Resetlink[B016]   = B016ResetLink;
  ms_Analyselink[B016] = B016AnalyseLink;
  ms_TestError[B016]   = B016TestError;
#endif
#endif /* SUN3OR4 */
    

#ifdef PC
#ifdef LNKb004
  ms_Available[B004] = 1;
  ms_Openlink[B004]    = B004OpenLink;
  ms_Closelink[B004]   = B004CloseLink;
  ms_Readlink[B004]    = B004ReadLink;
  ms_Writelink[B004]   = B004WriteLink;
  ms_Resetlink[B004]   = B004ResetLink;
  ms_Analyselink[B004] = B004AnalyseLink;
  ms_TestError[B004]   = B004TestError;
#endif

#ifdef LNKb008
  ms_Available[B008] = 1;
  ms_Openlink[B008]    = B008OpenLink;
  ms_Closelink[B008]   = B008CloseLink;
  ms_Readlink[B008]    = B008ReadLink;
  ms_Writelink[B008]   = B008WriteLink;
  ms_Resetlink[B008]   = B008ResetLink;
  ms_Analyselink[B008] = B008AnalyseLink;
  ms_TestError[B008]   = B008TestError;
#endif
#endif /* PC */

#ifdef VAX
#ifdef LNKqt0
  ms_Available[QT0] = 1;
  ms_Openlink[QT0]    = QT0OpenLink;
  ms_Closelink[QT0]   = QT0CloseLink;
  ms_Readlink[QT0]    = QT0ReadLink;
  ms_Writelink[QT0]   = QT0WriteLink;
  ms_Resetlink[QT0]   = QT0ResetLink;
  ms_Analyselink[QT0] = QT0AnalyseLink;
  ms_TestError[QT0]   = QT0TestError;
#endif
#endif /* VAX */

}
